@@ -25,6 +25,61 @@
  * The author of this code is Derek J Smithies
  *
  *  $Log: iax2ep.h,v $
+ *  Revision 1.17  2007/08/13 04:24:26  csoutheren
+ *  Normalise IAX2 answer logic
+ *
+ *  Revision 1.16  2007/08/09 22:15:44  dereksmithies
+ *  Formatting tidyup. No functional difference.
+ *
+ *  Revision 1.15  2007/08/03 01:24:06  dereksmithies
+ *  Tidyups so it compiles...
+ *
+ *  Revision 1.14  2007/08/02 23:25:07  dereksmithies
+ *  Rework iax2 handling of incoming calls. This should ensure that woomera/simpleopal etc
+ *  will correctly advise on receiving an incoming call.
+ *
+ *  Revision 1.13  2007/04/19 06:17:21  csoutheren
+ *  Fixes for precompiled headers with gcc
+ *
+ *  Revision 1.12  2007/03/13 00:32:16  csoutheren
+ *  Simple but messy changes to allow compile time removal of protocol
+ *  options such as H.450 and H.460
+ *  Fix MakeConnection overrides
+ *
+ *  Revision 1.11  2007/03/01 05:51:03  rjongbloed
+ *  Fixed backward compatibility of OnIncomingConnection() virtual
+ *    functions on various classes. If an old override returned PFalse
+ *    then it will now abort the call as it used to.
+ *
+ *  Revision 1.10  2007/01/24 04:00:55  csoutheren
+ *  Arrrghh. Changing OnIncomingConnection turned out to have a lot of side-effects
+ *  Added some pure viritual functions to prevent old code from breaking silently
+ *  New OpalEndpoint and OpalConnection descendants will need to re-implement
+ *  OnIncomingConnection. Sorry :)
+ *
+ *  Revision 1.9  2007/01/12 02:39:00  dereksmithies
+ *  Remove the notion of srcProcessors and dstProcessor lists from the ep.
+ *  Ensure that the connection looks after the callProcessor.
+ *
+ *  Revision 1.8  2007/01/09 03:32:23  dereksmithies
+ *  Tidy up and improve the close down process - make it more robust.
+ *  Alter level of several PTRACE statements. Add Terminate() method to transmitter and receiver.
+ *
+ *  Revision 1.7  2006/12/18 03:18:41  csoutheren
+ *  Messy but simple fixes
+ *    - Add access to SIP REGISTER timeout
+ *    - Ensure OpalConnection options are correctly progagated
+ *
+ *  Revision 1.6  2006/09/11 03:08:51  dereksmithies
+ *  Add fixes from Stephen Cook (sitiveni@gmail.com) for new patches to
+ *  improve call handling. Notably, IAX2 call transfer. Many thanks.
+ *  Thanks also to the Google summer of code for sponsoring this work.
+ *
+ *  Revision 1.5  2006/08/09 03:46:39  dereksmithies
+ *  Add ability to register to a remote Asterisk box. The iaxProcessor class is split
+ *  into a callProcessor and a regProcessor class.
+ *  Big thanks to Stephen Cook, (sitiveni@gmail.com) for this work.
+ *
  *  Revision 1.4  2005/08/26 03:07:38  dereksmithies
  *  Change naming convention, so all class names contain the string "IAX2"
  *
@@ -48,7 +103,9 @@
 #ifndef IAX_ENDPOINT_H
 #define IAX_ENDPOINT_H
 
+#ifndef _PTLIB_H
 #include <ptlib.h>
+#endif
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -56,9 +113,13 @@
 
 #include <opal/endpoint.h>
 #include <iax2/iax2con.h>
+#include <iax2/processor.h>
+#include <iax2/regprocessor.h>
+#include <iax2/specialprocessor.h>
 
 class IAX2Receiver;
 class IAX2Transmit;
+class IAX2Processor;
 
 /** A class to take frames from the transmitter, and disperse them to
     the appropriate IAX2Connection class.  This class calls a method in
@@ -100,10 +161,12 @@ public:
   PSyncPoint activate;
 
   /**Flag to indicate if this receiver thread should keep listening for network data */
-  BOOL           keepGoing;
+  PBoolean           keepGoing;
 };
 
 
+
+
 /** A class to manage global variables. There is one Endpoint per application. */
 class IAX2EndPoint : public OpalEndPoint
 {
@@ -127,7 +190,7 @@ class IAX2EndPoint : public OpalEndPoint
 
   The default behaviour does nothing.
   */
-  virtual BOOL NewIncomingConnection(
+  virtual PBoolean NewIncomingConnection(
     OpalTransport * transport  /// Transport connection came in on
   );
   
@@ -147,12 +210,12 @@ class IAX2EndPoint : public OpalEndPoint
      
      The proto field is optional when passed to a specific endpoint. If it
      is present, however, it must agree with the endpoints protocol name or
-     FALSE is returned.
+     PFalse is returned.
      
      This function usually returns almost immediately with the connection
      continuing to occur in a new background thread.
      
-     If FALSE is returned then the connection could not be established. For
+     If PFalse is returned then the connection could not be established. For
      example if a PSTN endpoint is used and the assiciated line is engaged
      then it may return immediately. Returning a non-NULL value does not
      mean that the connection will succeed, only that an attempt is being
@@ -160,10 +223,12 @@ class IAX2EndPoint : public OpalEndPoint
      
      The default behaviour is pure.
   */
-  virtual BOOL MakeConnection(
-			      OpalCall & call,        /// Owner of connection
-			      const PString & party,  /// Remote party to call
-			      void * userData = NULL  /// Arbitrary data to pass to connection
+  virtual PBoolean MakeConnection(
+			      OpalCall & call,          ///<  Owner of connection
+			      const PString & party,    ///<  Remote party to call
+			      void * userData = NULL,   ///<  Arbitrary data to pass to connection
+			      unsigned int options = 0, ///<  options to pass to connection
+			      OpalConnection::StringOptions * stringOptions = NULL
 			      );
   
   /**Create a connection for the IAX endpoint.
@@ -173,7 +238,8 @@ class IAX2EndPoint : public OpalEndPoint
 					   OpalCall & call,            /// Owner of connection
 					   const PString & token,      /// token used to identify connection
 					   void * userData,             /// User data for connection
-					   const PString & remoteParty /// Person we are calling.
+					   const PString & remoteParty, /// Url to call or is calling.
+             const PString & remotePartyName = PString::Empty() /// Name to call or is calling.
 					   );
   //@}
   
@@ -181,7 +247,7 @@ class IAX2EndPoint : public OpalEndPoint
   //@{
   /**Setup the Endpoint internval variables, which is called at program 
      startup.*/
-  BOOL Initialise();
+  PBoolean Initialise();
 
   /**Handle a received IAX frame. This may be a mini frame or full frame */
   virtual void IncomingEthernetFrame (IAX2Frame *frame);
@@ -191,16 +257,20 @@ class IAX2EndPoint : public OpalEndPoint
      frame. If the connection is gone, don't bother transmitting the
      frame. There are exceptins to this rule, such as when a hangup
      packet is sent (which is after the connections has died. */
-  BOOL ConectionForFrameIsAlive(IAX2Frame *f);
+  PBoolean ConectionForFrameIsAlive(IAX2Frame *f);
   
-  /**Request a new src call number, one that is different to all other src
-     call numbers for this invocation of the program.*/
-  int NextSrcCallNumber();
-    
+  /**Request a new source call number, one that is different to 
+     all other source call numbers for this program.  
+
+     @return P_MAX_INDEX  if there is no available call number,
+     or return a unique valid call number.
+     */
+  PINDEX NextSrcCallNumber(IAX2Processor * processor);
+      
   /**Write the token of all connections in the connectionsActive
      structure to the trace file */
   void ReportStoredConnections();
-
+  
   /**Report the port in use for IAX calls */
   WORD ListenPortNumber()  { return 4569; }
       
@@ -227,41 +297,56 @@ class IAX2EndPoint : public OpalEndPoint
   
   /**Set the password to some value */
   void SetPassword(PString newValue);
-    
-  /**Return True if a connection (which matches this Frame ) can be
-     found. This check is called prior to transmission of this
-     frame. */
-  BOOL ConnectionForFrameIsAlive(IAX2Frame *f);
-  
+
+  /**It is possible that a retransmitted frame has been in the transmit queue,
+     and while sitting there that frames sending connection has died.  Thus,
+     prior to transmission, call tis method.
+
+     @return True if a connection (which matches this Frame ) can be
+     found. */
+  PBoolean ConnectionForFrameIsAlive(IAX2Frame *f);
+ 
   /**Get out sequence number to use on status query frames*/
   PINDEX GetOutSequenceNumberForStatusQuery();
   
   /**We have an incoming call. Do we accept ? */
   void StartRinging(PString remoteCaller);
   
-  /**Call back for when a connection has closed */
-  void OnConnectionClose(IAX2Connection & con);
-
     /**Handle new incoming connection from listener.
        
-    A return value of TRUE indicates that the transport object should be
-    deleted by the caller. FALSE indicates that something else (eg the
+    A return value of PTrue indicates that the transport object should be
+    deleted by the caller. PFalse indicates that something else (eg the
     connection) has taken over responsibility for deleting the transport.
     
-    Well, that is true of Opal. In iax2, we do all the work of creating a new connection etc.
-    The transport arguement is ignore. In Iax2, this method is void, as no value is returned.
-    Further, in iax2, we process the incoming frame.
+    Well, that is true of Opal. In iax2, we do all the work of creating a new
+    connection etc.  The transport arguement is ignore. In Iax2, this method
+    is void, as no value is returned.  Further, in iax2, we process the
+    incoming frame.
     */
-  void NewIncomingConnection(IAX2Frame *f  /// Frame carrying the new request.
-			     );
+    void NewIncomingConnection(
+      IAX2Frame *f  /// Frame carrying the new request.
+		);
 
+    /**A call back function whenever a connection is established.
+       This indicates that a connection to an endpoint was established. This
+       usually occurs after OnConnected() and indicates that the connection
+       is both connected and has media flowing.
+      */
+    void OnEstablished(
+       OpalConnection & con
+    );
 
-  /**Call back for when a connections is established (we have received
-     the first media packet) */
-  void OnEstablished(OpalConnection & con);
+    /**Called whenever a connection instance is finished being used to
+       manage a call. We trap this callback to remove the connection
+       from this endpoints token table. Once we are done the token
+       table, we then call the generic OpalEndpoint::OnReleased
+       method. */
+    virtual void OnReleased(
+      OpalConnection & connection   ///<  Connection that was established
+    );
 
-   /**Get the data formats this endpoint is capable of operating.
-       This provides a list of media data format names that may be used by an
+   /**Get the data formats this endpoint is capable of operating.  This
+       provides a list of media data format names that may be used by an
        OpalMediaStream may be created by a connection from this endpoint.
 
        Note that a specific connection may not actually support all of the
@@ -269,7 +354,9 @@ class IAX2EndPoint : public OpalEndPoint
 
        The default behaviour is pure.
       */
-    virtual OpalMediaFormatList GetMediaFormats() const;
+  virtual OpalMediaFormatList GetMediaFormats() const;
+
+  virtual PBoolean OnIncomingCall(IAX2Connection & conn);
 
   /**Return the bitmask which specifies the possible codecs we support */
   PINDEX GetSupportedCodecs(OpalMediaFormatList & list);
@@ -302,21 +389,80 @@ class IAX2EndPoint : public OpalEndPoint
 
      pull the string apart and get the components. The compoents are stored
      in a PStringList, indexed by the enum RemoteAddressFields */
-  static PStringList DissectRemoteParty(const PString & other);
+  static PStringArray DissectRemoteParty(const PString & other);
 
   /**Pull frames off the incoming list, and pass on to the relevant
      connection. If no matching connection found, delete the frame.
      Repeat the process until no frames are left. */
   void ProcessReceivedEthernetFrames();
 
-
   /**Report on the frames in the current transmitter class, which are
      pending transmission*/
-  void ReportTransmitterLists();
+  void ReportTransmitterLists(PString & answer, bool getFullReport = false);
 
   /**Copy to the supplied OpalMediaList the media formats we support*/
   void CopyLocalMediaFormats(OpalMediaFormatList & list);
-    
+  
+  /**Register with a remote iax2 server.  The host can either be a 
+     hostname or ip address.  The password is optional as some servers
+     may not require it to register.  The requested refresh time is the
+     time that the registration should be refreshed in seconds.  The time
+     must be more than 10 seconds.*/
+  void Register(
+      const PString & host,
+      const PString & username,
+      const PString & password = PString::Empty(),
+      PINDEX requestedRefreshTime = 60
+    );
+  
+  enum RegisteredError {
+    RegisteredFailureUnknown
+  };
+  
+  /**This is a call back if an event related to registration occurs.
+     This callback should return as soon as possible.*/
+  virtual void OnRegistered(
+      const PString & host,
+      const PString & userName,
+      PBoolean isFailure,
+      RegisteredError reason = RegisteredFailureUnknown);
+   
+   /**Unregister from a registrar. This function is synchronous so it
+      will block.*/
+  void Unregister(
+      const PString & host,
+      const PString & username);
+      
+  enum UnregisteredError {
+    UnregisteredFailureUnknown
+  };
+      
+  /**This is a call back if an event related to unregistration occurs.
+     This callback should return as soon as possible.  Generally even if
+     a failure occurs when unregistering it should be ignored because it
+     does not matter to much that it couldn't unregister.*/
+  virtual void OnUnregistered(
+      const PString & host,
+      const PString & userName,
+      PBoolean isFailure,
+      UnregisteredError reason = UnregisteredFailureUnknown);
+      
+  
+  /**Check if an account is registered or being registered*/
+  PBoolean IsRegistered(const PString & host, const PString & username);
+  
+  /**Get the number of accounts that are being registered*/
+  PINDEX GetRegistrationsCount();
+  
+  /**Builds a url*/
+  PString BuildUrl(
+    const PString & host,
+    const PString & userName = PString::Empty(),
+    const PString & extension = PString::Empty(),
+    const PString & context = PString::Empty(),
+    const PString & transport = PString::Empty()
+  );
+  
   //@}
   
  protected:
@@ -361,24 +507,25 @@ class IAX2EndPoint : public OpalEndPoint
   
   /**Pointer to the Processor class which handles special packets (eg lagrq) that have no 
      destination call to handle them. */
-  IAX2Processor * specialPacketHandler;
+  IAX2SpecialProcessor * specialPacketHandler;
     
-  /**For the supplied IAX2Frame, pass it to a connection in the connectionsActive structure.
-     If no matching connection is found, return FALSE;
+  /**For the supplied IAX2Frame, pass it to a connection in the
+     connectionsActive structure.  If no matching connection is found, return
+     PFalse;
      
      If a matching connections is found, give the frame to the
-     connection (for the connection to process) and return TRUE;
+     connection (for the connection to process) and return PTrue;
   */
-  BOOL ProcessInMatchingConnection(IAX2Frame *f);  
-  
+  PBoolean ProcessInMatchingConnection(IAX2Frame *f);  
+    
   /**The TokenTranslationDict may need a new entry. Examine
      the list of active connections, to see if any match this frame.
      If any do, then we add a new translation entry in tokenTable;
      
      If a matching connection is found in connectionsActive, create a
-     new translation entry and return TRUE. The connection, after
+     new translation entry and return PTrue. The connection, after
      processing the frame, will then delete the frame. */
-  BOOL AddNewTranslationEntry(IAX2Frame *f);
+  PBoolean AddNewTranslationEntry(IAX2Frame *f);
   
   /**tokenTable is a hack to allow IAX2 to fit in with one of the
      demands of the opal library. 
@@ -394,8 +541,10 @@ class IAX2EndPoint : public OpalEndPoint
 
   PStringToString    tokenTable;
   
-  /**Threading mutex on the variable tokenTable   */
-  PMutex             mutexTokenTable;
+  /**Threading mutex on the variable tokenTable. We can now safely
+     read/write to this table, with the minimum of interference between
+     threads.  */
+  PReadWriteMutex    mutexTokenTable;
 
   /**Thread safe counter which keeps track of the calls created by this endpoint.
      This value is used when giving outgoing calls a unique ID */
@@ -403,16 +552,25 @@ class IAX2EndPoint : public OpalEndPoint
 
   /**Local copy of the media types we can handle*/
   OpalMediaFormatList localMediaFormats;
+  
+   /**A mutex to protect the registerProcessors collection*/
+  PMutex regProcessorsMutex;
+  
+  /**An array of register processors.  These are created when
+     another class calls register and deleted when another class
+     calls unregister or class destructor is called.  This collection
+     must be protected by the regProcessorsMutex*/
+  PArrayObjects regProcessors;
+  
 };
 
 #endif // IAX_ENDPOINT_H
 /* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
+/* With the comment below, the tab key does auto indent to 2 spaces.     */
 
 /*
  * Local Variables:
  * mode:c
- * c-file-style:linux
  * c-basic-offset:2
  * End:
  */
