@@ -24,114 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: mediastrm.h,v $
- * Revision 2.28.2.3  2007/01/15 22:16:42  dsandras
- * Backported patches improving stability from HEAD to Phobos.
- *
- * Revision 2.28.2.2  2006/12/08 06:27:20  csoutheren
- * Fix compilation problem caused by bad patch backports
- * Allow compilation with latest PWLib
- *
- * Revision 2.28.2.1  2006/08/07 20:26:27  dsandras
- * Backported fix from HEAD.
- *
- * Revision 2.28  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.27  2005/09/04 06:23:38  rjongbloed
- * Added OpalMediaCommand mechanism (via PNotifier) for media streams
- *   and media transcoders to send commands back to remote.
- *
- * Revision 2.26  2005/08/31 13:19:25  rjongbloed
- * Added mechanism for controlling media (especially codecs) including
- *   changing the OpalMediaFormat option list (eg bit rate) and a completely
- *   new OpalMediaCommand abstraction for things like video fast update.
- *
- * Revision 2.25  2005/08/20 07:35:22  rjongbloed
- * Set video RTP timestamps to value dirived from real time clock.
- *
- * Revision 2.24  2005/08/04 17:23:38  dsandras
- * Added function to determine if a stream is open or not.
- *
- * Revision 2.23  2005/04/10 20:48:30  dsandras
- * Added functions to put an OpalMediaStream on pause.
- *
- * Revision 2.22  2005/03/12 00:33:27  csoutheren
- * Fixed problems with STL compatibility on MSVC 6
- * Fixed problems with video streams
- * Thanks to Adrian Sietsma
- *
- * Revision 2.21  2004/12/04 16:35:50  dsandras
- * Added a function to get the PChannel back from the OpalMediaStream.
- *
- * Revision 2.20  2004/10/02 11:50:54  rjongbloed
- * Fixed RTP media stream so assures RTP session is open before starting.
- *
- * Revision 2.19  2004/08/14 07:56:29  rjongbloed
- * Major revision to utilise the PSafeCollection classes for the connections and calls.
- *
- * Revision 2.18  2004/05/17 13:24:18  rjongbloed
- * Added silence suppression.
- *
- * Revision 2.17  2004/03/11 06:54:27  csoutheren
- * Added ability to disable SIP or H.323 stacks
- *
- * Revision 2.16  2003/06/02 02:57:10  rjongbloed
- * Moved LID specific media stream class to LID source file.
- *
- * Revision 2.15  2003/04/16 02:30:21  robertj
- * Fixed comments on ReadData() and WriteData() functions.
- *
- * Revision 2.14  2003/03/17 10:26:59  robertj
- * Added video support.
- *
- * Revision 2.13  2002/11/10 11:33:17  robertj
- * Updated to OpenH323 v1.10.3
- *
- * Revision 2.12  2002/09/16 02:52:35  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.11  2002/04/15 08:47:42  robertj
- * Fixed problem with mismatched payload type being propagated.
- * Fixed correct setting of jitter buffer size in RTP media stream.
- *
- * Revision 2.10  2002/02/13 02:33:29  robertj
- * Added ability for media patch (and transcoders) to handle multiple RTP frames.
- * Removed media stream being descended from PChannel, not really useful.
- *
- * Revision 2.9  2002/02/11 07:39:15  robertj
- * Added media bypass for streams between compatible protocols.
- *
- * Revision 2.8  2002/01/22 05:10:58  robertj
- * Removed redundant code
- *
- * Revision 2.7  2002/01/22 05:09:00  robertj
- * Removed payload mismatch detection from RTP media stream.
- * Added function to get media patch from media stream.
- *
- * Revision 2.6  2002/01/14 02:27:32  robertj
- * Added ability to turn jitter buffer off in media stream to allow for patches
- *   that do not require it.
- *
- * Revision 2.5  2001/10/15 04:28:35  robertj
- * Added delayed start of media patch threads.
- *
- * Revision 2.4  2001/10/04 05:44:00  craigs
- * Changed to start media patch threads in Paused state
- *
- * Revision 2.3  2001/10/04 00:41:20  robertj
- * Removed GetMediaFormats() function as is not useful.
- *
- * Revision 2.2  2001/08/21 01:10:35  robertj
- * Fixed propagation of sound channel buffers through media stream.
- *
- * Revision 2.1  2001/08/01 05:51:47  robertj
- * Made OpalMediaFormatList class global to help with documentation.
- *
- * Revision 2.0  2001/07/27 15:48:24  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
+ * $Revision: 20745 $
+ * $Author: rjongbloed $
+ * $Date: 2008-08-28 02:20:52 +0200 (jeu 28 aoû 2008) $
  */
 
 #ifndef __OPAL_MEDIASTRM_H
@@ -141,17 +36,21 @@
 #pragma interface
 #endif
 
+#include <ptclib/delaychan.h>
 
 #include <opal/buildopts.h>
 #include <opal/mediafmt.h>
 #include <opal/mediacmd.h>
+#include <ptlib/safecoll.h>
+#include <ptclib/guid.h>
 
-#include <ptlib/sound.h>
-#include <ptlib/videoio.h>
 
 class RTP_Session;
 class OpalMediaPatch;
 class OpalLine;
+class OpalConnection;
+class OpalRTPConnection;
+class OpalMediaStatistics;
 
 
 /**This class describes a media stream as used in the OPAL system. A media
@@ -159,18 +58,19 @@ class OpalLine;
    entities. For example, data being sent to an RTP session over a network
    would be through a media stream.
   */
-class OpalMediaStream : public PObject
+class OpalMediaStream : public PSafeObject
 {
-    PCLASSINFO(OpalMediaStream, PObject);
+    PCLASSINFO(OpalMediaStream, PSafeObject);
   protected:
   /**@name Construction */
   //@{
     /**Construct a new media stream.
       */
     OpalMediaStream(
+      OpalConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource                        ///<  Is a source stream
+      bool isSource                        ///<  Is a source stream
     );
 
   public:
@@ -206,12 +106,13 @@ class OpalMediaStream : public PObject
     /**Update the media format. This can be used to adjust the
        parameters of a codec at run time. Note you cannot change the basic
        media format, eg change GSM0610 to G.711, only options for that
-       format, eg 6k3 mode to 5k3 mode in G.723.1.
+       format, eg 6k3 mode to 5k3 mode in G.723.1. If the formats are
+       different then a OpalMediaFormat::Merge() is performed.
 
        The default behaviour updates the mediaFormat member variable and
        pases the value on to the OpalMediaPatch.
       */
-    virtual BOOL UpdateMediaFormat(
+    virtual PBoolean UpdateMediaFormat(
       const OpalMediaFormat & mediaFormat  ///<  New media format
     );
 
@@ -221,7 +122,7 @@ class OpalMediaStream : public PObject
 
        The default behaviour passes the command on to the OpalMediaPatch.
       */
-    virtual BOOL ExecuteCommand(
+    virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
     );
 
@@ -238,28 +139,38 @@ class OpalMediaStream : public PObject
 
     /**Open the media stream using the media format.
 
-       The default behaviour simply sets the isOpen variable to TRUE.
+       The default behaviour simply sets the isOpen variable to true.
       */
-    virtual BOOL Open();
+    virtual PBoolean Open();
 
     /**Start the media stream.
 
        The default behaviour calls Resume() on the associated OpalMediaPatch
        thread if it was suspended.
       */
-    virtual BOOL Start();
+    virtual PBoolean Start();
 
     /**Close the media stream.
 
        The default does nothing.
       */
-    virtual BOOL Close();
+    virtual PBoolean Close();
+
+    /**Callback that is called on the source stream once the media patch has started.
+       The default behaviour calls OpalConnection::OnMediaPatchStart()
+      */
+    virtual void OnPatchStart();
+
+    /**Callback that is called on the source stream once the media patch has started.
+       The default behaviour calls OpalConnection::OnMediaPatchStop()
+      */
+    virtual void OnPatchStop();
 
     /**Write a list of RTP frames of data to the sink media stream.
        The default behaviour simply calls WritePacket() on each of the
        elements in the list.
       */
-    virtual BOOL WritePackets(
+    virtual PBoolean WritePackets(
       RTP_DataFrameList & packets
     );
 
@@ -268,7 +179,7 @@ class OpalMediaStream : public PObject
        RTP_DataFrame and sets the frames timestamp and marker from the internal
        member variables of the media stream class.
       */
-    virtual BOOL ReadPacket(
+    virtual PBoolean ReadPacket(
       RTP_DataFrame & packet
     );
 
@@ -277,7 +188,7 @@ class OpalMediaStream : public PObject
        RTP_DataFrame and and sets the internal timestamp and marker from the
        member variables of the media stream class.
       */
-    virtual BOOL WritePacket(
+    virtual PBoolean WritePacket(
       RTP_DataFrame & packet
     );
 
@@ -286,7 +197,7 @@ class OpalMediaStream : public PObject
        RTP_DataFrame and sets the frames timestamp and marker from the internal
        member variables of the media stream class.
       */
-    virtual BOOL ReadData(
+    virtual PBoolean ReadData(
       BYTE * data,      ///<  Data buffer to read to
       PINDEX size,      ///<  Size of buffer
       PINDEX & length   ///<  Length of data actually read
@@ -297,18 +208,24 @@ class OpalMediaStream : public PObject
        RTP_DataFrame and and sets the internal timestamp and marker from the
        member variables of the media stream class.
       */
-    virtual BOOL WriteData(
+    virtual PBoolean WriteData(
       const BYTE * data,   ///<  Data to write
       PINDEX length,       ///<  Length of data to read.
       PINDEX & written     ///<  Length of data actually written
     );
 
+    /**Pushes a frame to the patch
+      */
+    bool PushPacket(
+      RTP_DataFrame & packet
+    );
+
     /**Set the data size in bytes that is expected to be used. Some media
        streams can make use of this information to perform optimisations.
 
        The default behaviour does nothing.
       */
-    virtual BOOL SetDataSize(
+    virtual PBoolean SetDataSize(
       PINDEX dataSize  ///<  New data size
     );
 
@@ -318,17 +235,26 @@ class OpalMediaStream : public PObject
     PINDEX GetDataSize() const { return defaultDataSize; }
 
     /**Indicate if the media stream is synchronous.
-       If this returns TRUE then the media stream will block of the amount of
+       If this returns true then the media stream will block of the amount of
        time it takes to annunciate the data. For example if the media stream
        is over a sound card, and 480 bytes of data are to be written it will
        take 30 milliseconds to complete.
       */
-    virtual BOOL IsSynchronous() const = 0;
+    virtual PBoolean IsSynchronous() const = 0;
 
-    /**Indicate if the media stream requires a OpalMediaPatch thread.
-       The default behaviour returns TRUE.
+    /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
+       This is called on the source stream and is passed the sink stream that the
+       patch will initially be using. The function could conditionally require
+       the patch thread to execute a thread reading and writing data, or prevent
+       it from doing so as it can do so in hardware in some way, e.g. if both
+       streams where on the same OpalLineInterfaceDevice.
+
+       The default behaviour simply returns true.
       */
-    virtual BOOL RequiresPatchThread() const;
+    virtual PBoolean RequiresPatchThread(
+      OpalMediaStream * sinkStream  ///< Sink stream for this source
+    ) const;
+    virtual PBoolean RequiresPatchThread() const; // For backward compatibility
 
     /**Enable jitter buffer for the media stream.
 
@@ -339,18 +265,27 @@ class OpalMediaStream : public PObject
 
   /**@name Member variable access */
   //@{
-    /**Determine of media stream is a source or a sink.
-      */
-    BOOL IsSource() const { return isSource; }
+    /** Get the owner connection.
+     */
+    OpalConnection & GetConnection() const { return connection; }
 
     /**Determine of media stream is a source or a sink.
       */
-    BOOL IsSink() const { return !isSource; }
+    bool IsSource() const { return isSource; }
+
+    /**Determine of media stream is a source or a sink.
+      */
+    bool IsSink() const { return !isSource; }
 
     /**Get the session number of the stream.
      */
     unsigned GetSessionID() const { return sessionID; }
 
+    /**  Get the ID associated with this stream. Used for detecting two 
+      *  the streams associated with a bidirectional media channel
+      */
+    PString GetID() const { return identifier; }
+
     /**Get the timestamp of last read.
       */
     unsigned GetTimestamp() const { return timestamp; }
@@ -361,34 +296,44 @@ class OpalMediaStream : public PObject
 
     /**Get the marker bit of last read.
       */
-    BOOL GetMarker() const { return marker; }
+    bool GetMarker() const { return marker; }
 
     /**Set marker bit for next write.
       */
-    void SetMarker(BOOL m) { marker = m; }
+    void SetMarker(bool m) { marker = m; }
 
-    /**Get the paused state for writing.
+    /**Get the paused state for stream.
       */
-    BOOL IsPaused() const { return paused; }
+    bool IsPaused() const { return paused; }
 
-    /**Set the paused state for writing.
+    /**Set the paused state for stream.
+       This will stop reading/writing data from the stream.
       */
-    void SetPaused(BOOL p) { paused = p; }
+    virtual void SetPaused(
+      bool pause    ///< Indicate that the stream should be paused
+    );
 
-    /**Returns TRUE if the media stream is open.
+    /**Returns true if the media stream is open.
       */
-    BOOL IsOpen() { return isOpen; }
+    bool IsOpen() { return isOpen; }
     
     /**Set the patch thread that is using this stream.
       */
-    void SetPatch(
+    virtual PBoolean SetPatch(
       OpalMediaPatch * patch  ///<  Media patch thread
     );
 
+    /**Remove the patch thread that is using this stream.
+       This function is useful in case of streams which can be accessed by
+       multiple instances of OpalMediaPatch.
+
+       The default behaviour simply sets patchThread to NULL.
+    */
+    virtual void RemovePatch(OpalMediaPatch * patch);
+
     /**Get the patch thread that is using the stream.
       */
-    OpalMediaPatch * GetPatch() const { return patchThread; }
-  //@}
+    OpalMediaPatch * GetPatch() const { return mediaPatch; }
 
     /**Add a filter to the owning patch safely.
       */
@@ -396,36 +341,61 @@ class OpalMediaStream : public PObject
 
     /**Remove a filter from the owning patch safely.
       */
-    BOOL RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
+    bool RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
 
-    /**Get the mutex that prevents the destructor from completing.
-      */
-    PMutex &  GetDeleteMutex() const { return deleteMutex; }
+#ifdef OPAL_STATISTICS
+    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+#endif
+  //@}
 
   protected:
-    OpalMediaFormat mediaFormat;
-    unsigned        sessionID;
-    BOOL	    paused;
-    BOOL            isSource;
-    BOOL            isOpen;
-    PINDEX          defaultDataSize;
-    unsigned        timestamp;
-    BOOL            marker;
-    unsigned        mismatchedPayloadTypes;
+    OpalConnection & connection;
+    unsigned         sessionID;
+    PString          identifier;
+    OpalMediaFormat  mediaFormat;
+    bool             paused;
+    bool             isSource;
+    bool             isOpen;
+    PINDEX           defaultDataSize;
+    unsigned         timestamp;
+    bool             marker;
+    unsigned         mismatchedPayloadTypes;
 
-    OpalMediaPatch * patchThread;
-    PMutex           patchMutex;
+    OpalMediaPatch * mediaPatch;
     PNotifier        commandNotifier;
-
-    mutable PMutex  deleteMutex;
 };
 
-PLIST(OpalMediaStreamList, OpalMediaStream);
+typedef PSafePtr<OpalMediaStream> OpalMediaStreamPtr;
+
+
+/**This is a helper class to delay the right time for non I/O bound streams.
+  */
+class OpalMediaStreamPacing
+{
+  public:
+    OpalMediaStreamPacing(
+      const OpalMediaFormat & mediaFormat ///<  Media format for stream
+    );
+
+    /// Delay appropriate time for the written bytes
+    void Pace(
+      bool reading,     ///< Are reading from medium
+      PINDEX bytes,     ///< Bytes read/written
+      bool & marker     ///< RTP Marker
+    );
+
+  protected:
+    bool           m_isAudio;
+    unsigned       m_frameTime;
+    PINDEX         m_frameSize;
+    unsigned       m_timeUnits;
+    PAdaptiveDelay m_delay;
+};
 
 
 /**This class describes a media stream that is used for media bypass.
   */
-class OpalNullMediaStream : public OpalMediaStream
+class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
 {
     PCLASSINFO(OpalNullMediaStream, OpalMediaStream);
   public:
@@ -434,42 +404,47 @@ class OpalNullMediaStream : public OpalMediaStream
     /**Construct a new media stream for RTP sessions.
       */
     OpalNullMediaStream(
+      OpalConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource                        ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
+      bool isSynchronous = false           ///<  Can accept data and block accordingly
     );
   //@}
 
   /**@name Overrides of OpalMediaStream class */
   //@{
     /**Read raw media data from the source media stream.
-       The default behaviour does nothing and returns FALSE.
+       The default behaviour does nothing and returns false.
       */
-    virtual BOOL ReadData(
+    virtual PBoolean ReadData(
       BYTE * data,      ///<  Data buffer to read to
       PINDEX size,      ///<  Size of buffer
       PINDEX & length   ///<  Length of data actually read
     );
 
     /**Write raw media data to the sink media stream.
-       The default behaviour does nothing and returns FALSE.
+       The default behaviour does nothing and returns false.
       */
-    virtual BOOL WriteData(
+    virtual PBoolean WriteData(
       const BYTE * data,   ///<  Data to write
       PINDEX length,       ///<  Length of data to read.
       PINDEX & written     ///<  Length of data actually written
     );
-
-    /**Indicate if the media stream requires a OpalMediaPatch thread.
-       The default behaviour returns FALSE.
+	
+    /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
+       The default behaviour returns false.
       */
-    virtual BOOL RequiresPatchThread() const;
+    virtual PBoolean RequiresPatchThread() const;
 
     /**Indicate if the media stream is synchronous.
-       Returns FALSE.
+       Returns false.
       */
-    virtual BOOL IsSynchronous() const;
+    virtual PBoolean IsSynchronous() const;
   //@}
+
+  protected:
+    bool           m_isSynchronous;
 };
 
 
@@ -483,54 +458,61 @@ class OpalRTPMediaStream : public OpalMediaStream
   /**@name Construction */
   //@{
     /**Construct a new media stream for RTP sessions.
+       This will add a reference to the rtpSession passed in.
       */
     OpalRTPMediaStream(
+      OpalRTPConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
-      BOOL isSource,                       ///<  Is a source stream
-      RTP_Session & rtpSession,    ///<  RTP session to stream to/from
-      unsigned minAudioJitterDelay,///<  Minimum jitter buffer size (if applicable)
-      unsigned maxAudioJitterDelay ///<  Maximum jitter buffer size (if applicable)
+      bool isSource,                       ///<  Is a source stream
+      RTP_Session & rtpSession,            ///<  RTP session to stream to/from
+      unsigned minAudioJitterDelay,        ///<  Minimum jitter buffer size (if applicable)
+      unsigned maxAudioJitterDelay         ///<  Maximum jitter buffer size (if applicable)
     );
+
+    /**Destroy the media stream for RTP sessions.
+       This will release the reference to the rtpSession passed into the constructor.
+      */
+    ~OpalRTPMediaStream();
   //@}
 
   /**@name Overrides of OpalMediaStream class */
   //@{
     /**Open the media stream using the media format.
 
-       The default behaviour simply sets the isOpen variable to TRUE.
+       The default behaviour simply sets the isOpen variable to true.
       */
-    virtual BOOL Open();
+    virtual PBoolean Open();
 
     /**Close the media stream.
 
        The default does nothing.
       */
-    virtual BOOL Close();
+    virtual PBoolean Close();
 
     /**Read an RTP frame of data from the source media stream.
        The new behaviour simply calls RTP_Session::ReadData().
       */
-    virtual BOOL ReadPacket(
+    virtual PBoolean ReadPacket(
       RTP_DataFrame & packet
     );
 
     /**Write an RTP frame of data to the sink media stream.
        The new behaviour simply calls RTP_Session::WriteData().
       */
-    virtual BOOL WritePacket(
+    virtual PBoolean WritePacket(
       RTP_DataFrame & packet
     );
 
     /**Set the data size in bytes that is expected to be used.
       */
-    virtual BOOL SetDataSize(
+    virtual PBoolean SetDataSize(
       PINDEX dataSize  ///<  New data size
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns FALSE for RTP streams.
+       Returns false for RTP streams.
       */
-    virtual BOOL IsSynchronous() const;
+    virtual PBoolean IsSynchronous() const;
 
     /**Enable jitter buffer for the media stream.
 
@@ -543,6 +525,9 @@ class OpalRTPMediaStream : public OpalMediaStream
     virtual RTP_Session & GetRtpSession() const
     { return rtpSession; }
 
+#ifdef OPAL_STATISTICS
+    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+#endif
   //@}
 
   protected:
@@ -564,11 +549,12 @@ class OpalRawMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalRawMediaStream(
+      OpalConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource,                       ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       PChannel * channel,                  ///<  I/O channel to stream to/from
-      BOOL autoDelete                      ///<  Automatically delete channel
+      bool autoDelete                      ///<  Automatically delete channel
     );
 
     /**Delete attached channel if autoDelete enabled.
@@ -582,7 +568,7 @@ class OpalRawMediaStream : public OpalMediaStream
     /**Read raw media data from the source media stream.
        The default behaviour reads from the PChannel object.
       */
-    virtual BOOL ReadData(
+    virtual PBoolean ReadData(
       BYTE * data,      ///<  Data buffer to read to
       PINDEX size,      ///<  Size of buffer
       PINDEX & length   ///<  Length of data actually read
@@ -591,7 +577,7 @@ class OpalRawMediaStream : public OpalMediaStream
     /**Write raw media data to the sink media stream.
        The default behaviour writes to the PChannel object.
       */
-    virtual BOOL WriteData(
+    virtual PBoolean WriteData(
       const BYTE * data,   ///<  Data to write
       PINDEX length,       ///<  Length of data to read.
       PINDEX & written     ///<  Length of data actually written
@@ -605,7 +591,7 @@ class OpalRawMediaStream : public OpalMediaStream
 
        Closes the associated PChannel.
       */
-    virtual BOOL Close();
+    virtual PBoolean Close();
 
     /**Get average signal level in last frame.
       */
@@ -614,11 +600,12 @@ class OpalRawMediaStream : public OpalMediaStream
 
   protected:
     PChannel * channel;
-    PMutex     channel_mutex;
-    BOOL       autoDelete;
+    bool       autoDelete;
 
     PUInt64    averageSignalSum;
     unsigned   averageSignalSamples;
+    PMutex     averagingMutex;
+
     void CollectAverage(const BYTE * buffer, PINDEX size);
 };
 
@@ -626,7 +613,7 @@ class OpalRawMediaStream : public OpalMediaStream
 
 /**This class describes a media stream that transfers data to/from a file.
   */
-class OpalFileMediaStream : public OpalRawMediaStream
+class OpalFileMediaStream : public OpalRawMediaStream, public OpalMediaStreamPacing
 {
     PCLASSINFO(OpalFileMediaStream, OpalRawMediaStream);
   public:
@@ -635,19 +622,21 @@ class OpalFileMediaStream : public OpalRawMediaStream
     /**Construct a new media stream for files.
       */
     OpalFileMediaStream(
+      OpalConnection &,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource,                       ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       PFile * file,                        ///<  File to stream to/from
-      BOOL autoDelete = TRUE               ///<  Automatically delete file
+      bool autoDelete = true               ///<  Automatically delete file
     );
 
     /**Construct a new media stream for files.
       */
     OpalFileMediaStream(
+      OpalConnection & ,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource,                       ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       const PFilePath & path               ///<  File path to stream to/from
     );
   //@}
@@ -655,21 +644,38 @@ class OpalFileMediaStream : public OpalRawMediaStream
   /**@name Overrides of OpalMediaStream class */
   //@{
     /**Indicate if the media stream is synchronous.
-       Returns TRUE for LID streams.
+       Returns true for LID streams.
       */
-    virtual BOOL IsSynchronous() const;
+    virtual PBoolean IsSynchronous() const;
   //@}
 
+    virtual PBoolean ReadData(
+      BYTE * data,      ///<  Data buffer to read to
+      PINDEX size,      ///<  Size of buffer
+      PINDEX & length   ///<  Length of data actually read
+    );
+
+    /**Write raw media data to the sink media stream.
+       The default behaviour writes to the PChannel object.
+      */
+    virtual PBoolean WriteData(
+      const BYTE * data,   ///<  Data to write
+      PINDEX length,       ///<  Length of data to read.
+      PINDEX & written     ///<  Length of data actually written
+    );
+
   protected:
     PFile file;
 };
 
-class OpalMediaFormat;
-class PSoundChannel;
+
+#if P_AUDIO
 
 /**This class describes a media stream that transfers data to/from a audio
    PSoundChannel.
   */
+class PSoundChannel;
+
 class OpalAudioMediaStream : public OpalRawMediaStream
 {
     PCLASSINFO(OpalAudioMediaStream, OpalRawMediaStream);
@@ -679,20 +685,22 @@ class OpalAudioMediaStream : public OpalRawMediaStream
     /**Construct a new media stream for audio.
       */
     OpalAudioMediaStream(
+      OpalConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource,                       ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       PINDEX buffers,                      ///<  Number of buffers on sound channel
       PSoundChannel * channel,             ///<  Audio device to stream to/from
-      BOOL autoDelete = TRUE               ///<  Automatically delete PSoundChannel
+      bool autoDelete = true               ///<  Automatically delete PSoundChannel
     );
 
     /**Construct a new media stream for audio.
       */
     OpalAudioMediaStream(
+      OpalConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
-      BOOL isSource,                       ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       PINDEX buffers,                      ///<  Number of buffers on sound channel
       const PString & deviceName           ///<  Name of audio device to stream to/from
     );
@@ -705,24 +713,30 @@ class OpalAudioMediaStream : public OpalRawMediaStream
 
        The defafault simply sets teh member variable defaultDataSize.
       */
-    virtual BOOL SetDataSize(
+    virtual PBoolean SetDataSize(
       PINDEX dataSize  ///<  New data size
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns TRUE for LID streams.
+       Returns true for LID streams.
       */
-    virtual BOOL IsSynchronous() const;
+    virtual PBoolean IsSynchronous() const;
   //@}
 
   protected:
     PINDEX soundChannelBuffers;
 };
 
+#endif // P_AUDIO
+
+#if OPAL_VIDEO
 
 /**This class describes a media stream that transfers data to/from a
    PVideoDevice.
   */
+class PVideoInputDevice;
+class PVideoOutputDevice;
+
 class OpalVideoMediaStream : public OpalMediaStream
 {
     PCLASSINFO(OpalVideoMediaStream, OpalMediaStream);
@@ -732,11 +746,12 @@ class OpalVideoMediaStream : public OpalMediaStream
     /**Construct a new media stream for channel.
       */
     OpalVideoMediaStream(
+      OpalConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       PVideoInputDevice * inputDevice,     ///<  Device to use for video grabbing
       PVideoOutputDevice * outputDevice,   ///<  Device to use for video display
-      BOOL autoDelete = TRUE               ///<  Automatically delete PVideoDevices
+      bool autoDelete = true               ///<  Automatically delete PVideoDevices
     );
 
     /**Delete attached channel if autoDelete enabled.
@@ -751,14 +766,20 @@ class OpalVideoMediaStream : public OpalMediaStream
        The default behaviour sets the OpalLineInterfaceDevice format and
        calls Resume() on the associated OpalMediaPatch thread.
       */
-    virtual BOOL Open();
+    virtual PBoolean Open();
+
+    /**Close the media stream.
+
+       The default does nothing.
+      */
+    virtual PBoolean Close();
 
     /**Read raw media data from the source media stream.
        The default behaviour simply calls ReadPacket() on the data portion of the
        RTP_DataFrame and sets the frames timestamp and marker from the internal
        member variables of the media stream class.
       */
-    virtual BOOL ReadData(
+    virtual PBoolean ReadData(
       BYTE * data,      ///<  Data buffer to read to
       PINDEX size,      ///<  Size of buffer
       PINDEX & length   ///<  Length of data actually read
@@ -769,32 +790,99 @@ class OpalVideoMediaStream : public OpalMediaStream
        RTP_DataFrame and and sets the internal timestamp and marker from the
        member variables of the media stream class.
       */
-    virtual BOOL WriteData(
+    virtual PBoolean WriteData(
       const BYTE * data,   ///<  Data to write
       PINDEX length,       ///<  Length of data to read.
       PINDEX & written     ///<  Length of data actually written
     );
 
     /**Indicate if the media stream is synchronous.
-       Returns TRUE for LID streams.
+       Returns true for LID streams.
       */
-    virtual BOOL IsSynchronous() const;
+    virtual PBoolean IsSynchronous() const;
 
     /** Override size of frame header is included
       */
-    virtual BOOL SetDataSize(
+    virtual PBoolean SetDataSize(
      PINDEX dataSize  ///<  New data size
     );
 
+    /** Get the input device (e.g. for statistics)
+      */
+    virtual PVideoInputDevice * GetVideoInputDevice() const {
+      return inputDevice;
+    }
+
+    /** Get the output device (e.g. for statistics)
+      */
+    virtual PVideoOutputDevice * GetVideoOutputDevice() const {
+      return outputDevice;
+    }
+
   //@}
 
   protected:
     PVideoInputDevice  * inputDevice;
     PVideoOutputDevice * outputDevice;
-    BOOL                 autoDelete;
+    bool                 autoDelete;
     PTimeInterval        lastGrabTime;
 };
 
+#endif // OPAL_VIDEO
+
+class OpalTransportUDP;
+
+/** Media stream that uses UDP.
+ */
+class OpalUDPMediaStream : public OpalMediaStream
+{
+    PCLASSINFO(OpalUDPMediaStream, OpalMediaStream);
+  public:
+  /**@name Construction */
+  //@{
+    /**Construct a new media stream for channel.
+      */
+    OpalUDPMediaStream(
+      OpalConnection & conn,
+      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
+      unsigned sessionID,                  ///<  Session number for stream
+      bool isSource,                       ///<  Is a source stream
+      OpalTransportUDP & transport         ///<  UDP transport instance
+    );
+  //@}
+
+  /**@name Overrides of OpalMediaStream class */
+  //@{
+
+    /**Read an RTP frame of data from the source media stream.
+       The new behaviour simply calls OpalTransportUDP::ReadPDU().
+      */
+    virtual PBoolean ReadPacket(
+      RTP_DataFrame & packet
+    );
+
+    /**Write an RTP frame of data to the sink media stream.
+       The new behaviour simply calls OpalTransportUDP::Write().
+      */
+    virtual PBoolean WritePacket(
+      RTP_DataFrame & packet
+    );
+
+    /**Indicate if the media stream is synchronous.
+       Returns false.
+      */
+    virtual PBoolean IsSynchronous() const;
+
+    /**Close the media stream.
+       Closes the associated OpalTransportUDP.
+      */
+    virtual PBoolean Close();
+
+  //@}
+
+  private:
+    OpalTransportUDP & udpTransport;
+};
 
 
 #endif //__OPAL_MEDIASTRM_H
