@@ -23,139 +23,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: h323rtp.cxx,v $
- * Revision 2.12  2005/03/03 18:31:23  dsandras
- * Fixed silence detection definition in the logical channel parameters.
- *
- * Revision 2.11  2004/02/24 11:28:46  rjongbloed
- * Normalised RTP session management across protocols
- *
- * Revision 2.10  2004/02/19 10:47:04  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 2.9  2003/01/07 04:39:53  robertj
- * Updated to OpenH323 v1.11.2
- *
- * Revision 2.8  2002/11/10 11:33:19  robertj
- * Updated to OpenH323 v1.10.3
- *
- * Revision 2.7  2002/09/04 06:01:49  robertj
- * Updated to OpenH323 v1.9.6
- *
- * Revision 2.6  2002/07/01 04:56:32  robertj
- * Updated to OpenH323 v1.9.1
- *
- * Revision 2.5  2002/01/22 05:26:39  robertj
- * Added enum for illegal payload type value.
- *
- * Revision 2.4  2001/12/07 08:53:30  robertj
- * Renamed RTP to be more general UDP port base.
- *
- * Revision 2.3  2001/11/12 05:32:12  robertj
- * Added OpalTransportAddress::GetIpAddress when don't need port number.
- *
- * Revision 2.2  2001/10/05 00:22:14  robertj
- * Updated to PWLib 1.2.0 and OpenH323 1.7.0
- *
- * Revision 2.1  2001/08/13 05:10:40  robertj
- * Updates from OpenH323 v1.6.0 release.
- *
- * Revision 2.0  2001/07/27 15:48:25  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
- * Revision 1.27  2003/10/27 06:03:39  csoutheren
- * Added support for QoS
- *   Thanks to Henry Harrison of AliceStreet
- *
- * Revision 1.26  2003/02/07 00:28:24  robertj
- * Changed function to virtual to help in using external multiicast RTP stacks.
- *
- * Revision 1.25  2003/02/05 06:32:10  robertj
- * Fixed non-stun symmetric NAT support recently broken.
- *
- * Revision 1.24  2003/02/05 01:55:14  robertj
- * Fixed setting of correct address in OLC's when STUN is used.
- *
- * Revision 1.23  2003/02/04 07:06:41  robertj
- * Added STUN support.
- *
- * Revision 1.22  2002/11/19 01:47:26  robertj
- * Included canonical name in RTP statistics returned in IRR
- *
- * Revision 1.21  2002/10/08 13:08:21  robertj
- * Changed for IPv6 support, thanks Sbastien Josset.
- *
- * Revision 1.20  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.19  2002/07/02 10:02:32  robertj
- * Added H323TransportAddress::GetIpAddress() so don't have to provide port
- *   when you don't need it as in GetIpAndPort(),.
- *
- * Revision 1.18  2002/06/24 08:07:49  robertj
- * Fixed setting of H.225.0 logical channel parameter silenceSuppression field
- *   to correctly indicate if codec is going to stop sending RTP on silence.
- *
- * Revision 1.17  2002/05/28 06:27:23  robertj
- * Split UDP (for RAS) from RTP port bases.
- * Added current port variable so cycles around the port range specified which
- *   fixes some wierd problems on some platforms, thanks Federico Pinna
- *
- * Revision 1.16  2001/10/02 02:06:23  robertj
- * Fixed CIsco IOS compatibility, yet again!.
- *
- * Revision 1.15  2001/10/02 01:53:53  robertj
- * Fixed CIsco IOS compatibility, again.
- *
- * Revision 1.14  2001/08/06 03:08:57  robertj
- * Fission of h323.h to h323ep.h & h323con.h, h323.h now just includes files.
- *
- * Revision 1.13  2001/02/09 05:13:56  craigs
- * Added pragma implementation to (hopefully) reduce the executable image size
- * under Linux
- *
- * Revision 1.12  2001/01/25 07:27:16  robertj
- * Major changes to add more flexible OpalMediaFormat class to normalise
- *   all information about media types, especially codecs.
- *
- * Revision 1.11  2000/12/18 08:59:20  craigs
- * Added ability to set ports
- *
- * Revision 1.10  2000/09/22 00:32:34  craigs
- * Added extra logging
- * Fixed problems with no fastConnect with tunelling
- *
- * Revision 1.9  2000/08/31 08:15:41  robertj
- * Added support for dynamic RTP payload types in H.245 OpenLogicalChannel negotiations.
- *
- * Revision 1.8  2000/08/23 14:27:04  craigs
- * Added prototype support for Microsoft GSM codec
- *
- * Revision 1.7  2000/07/12 13:06:49  robertj
- * Removed test for sessionID in OLC, just trace a warning instead of abandoning connection.
- *
- * Revision 1.6  2000/07/11 19:36:43  robertj
- * Fixed silenceSuppression field in OLC only to be included on transmitter.
- *
- * Revision 1.5  2000/05/23 12:57:37  robertj
- * Added ability to change IP Type Of Service code from applications.
- *
- * Revision 1.4  2000/05/02 04:32:27  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.3  2000/04/05 03:17:32  robertj
- * Added more RTP statistics gathering and H.245 round trip delay calculation.
- *
- * Revision 1.2  2000/01/20 05:57:46  robertj
- * Added extra flexibility in receiving incorrectly formed OpenLogicalChannel PDU's
- *
- * Revision 1.1  1999/12/23 23:02:36  robertj
- * File reorganision for separating RTP from H.323 and creation of LID for VPB support.
- *
+ * $Revision: 19717 $
+ * $Author: rjongbloed $
+ * $Date: 2008-03-11 03:44:12 +0100 (mar 11 mar 2008) $
  */
 
 #include <ptlib.h>
 
+#include <opal/buildopts.h>
+#if OPAL_H323
+
 #ifdef __GNUC__
 #pragma implementation "h323rtp.h"
 #endif
@@ -165,6 +42,7 @@
 #include <opal/manager.h>
 #include <h323/h323ep.h>
 #include <h323/h323con.h>
+#include <h323/h323pdu.h>
 #include <h323/transaddr.h>
 #include <asn/h225.h>
 #include <asn/h245.h>
@@ -203,7 +81,7 @@ H323_RTP_UDP::H323_RTP_UDP(const H323Connection & conn,
 }
 
 
-BOOL H323_RTP_UDP::OnSendingPDU(const H323_RTPChannel & channel,
+PBoolean H323_RTP_UDP::OnSendingPDU(const H323_RTPChannel & channel,
                                 H245_H2250LogicalChannelParameters & param) const
 {
   PTRACE(3, "RTP\tOnSendingPDU");
@@ -211,7 +89,7 @@ BOOL H323_RTP_UDP::OnSendingPDU(const H323_RTPChannel & channel,
   param.m_sessionID = rtp.GetSessionID();
 
   param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaGuaranteedDelivery);
-  param.m_mediaGuaranteedDelivery = FALSE;
+  param.m_mediaGuaranteedDelivery = PFalse;
 
   // unicast must have mediaControlChannel
   H323TransportAddress mediaControlAddress(rtp.GetLocalAddress(), rtp.GetLocalControlPort());
@@ -226,7 +104,7 @@ BOOL H323_RTP_UDP::OnSendingPDU(const H323_RTPChannel & channel,
   }
   else {
     // Set flag for we are going to stop sending audio on silence
-    OpalMediaStream * mediaStream = channel.GetMediaStream();
+    OpalMediaStreamPtr mediaStream = channel.GetMediaStream();
     if (mediaStream != NULL) {
       param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_silenceSuppression);
       param.m_silenceSuppression = (connection.GetEndPoint ().GetManager ().GetSilenceDetectParams ().m_mode != OpalSilenceDetector::NoSilenceDetection);
@@ -234,13 +112,18 @@ BOOL H323_RTP_UDP::OnSendingPDU(const H323_RTPChannel & channel,
   }
 
   // Set dynamic payload type, if is one
-  int rtpPayloadType = channel.GetDynamicRTPPayloadType();
+  RTP_DataFrame::PayloadTypes rtpPayloadType = channel.GetDynamicRTPPayloadType();
   if (rtpPayloadType >= RTP_DataFrame::DynamicBase && rtpPayloadType < RTP_DataFrame::IllegalPayloadType) {
     param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_dynamicRTPPayloadType);
-    param.m_dynamicRTPPayloadType = rtpPayloadType;
+    param.m_dynamicRTPPayloadType = (int)rtpPayloadType;
   }
 
-  return TRUE;
+  // Set the media packetization field if have an option to describe it.
+  param.m_mediaPacketization.SetTag(H245_H2250LogicalChannelParameters_mediaPacketization::e_rtpPayloadType);
+  if (H323SetRTPPacketization(param.m_mediaPacketization, channel.GetMediaStream()->GetMediaFormat(), rtpPayloadType))
+    param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaPacketization);
+
+  return PTrue;
 }
 
 
@@ -268,70 +151,81 @@ void H323_RTP_UDP::OnSendingAckPDU(const H323_RTPChannel & channel,
 }
 
 
-BOOL H323_RTP_UDP::ExtractTransport(const H245_TransportAddress & pdu,
-                                    BOOL isDataPort,
+PBoolean H323_RTP_UDP::ExtractTransport(const H245_TransportAddress & pdu,
+                                    PBoolean isDataPort,
                                     unsigned & errorCode)
 {
   if (pdu.GetTag() != H245_TransportAddress::e_unicastAddress) {
     PTRACE(1, "RTP_UDP\tOnly unicast supported at this time");
     errorCode = H245_OpenLogicalChannelReject_cause::e_multicastChannelNotAllowed;
-    return FALSE;
+    return PFalse;
   }
 
   H323TransportAddress transAddr = pdu;
 
   PIPSocket::Address ip;
-  WORD port;
+  WORD port = 0;
   if (transAddr.GetIpAndPort(ip, port))
     return rtp.SetRemoteSocketInfo(ip, port, isDataPort);
 
-  return FALSE;
+  return PFalse;
 }
 
 
-BOOL H323_RTP_UDP::OnReceivedPDU(H323_RTPChannel & channel,
+PBoolean H323_RTP_UDP::OnReceivedPDU(H323_RTPChannel & channel,
                                  const H245_H2250LogicalChannelParameters & param,
                                  unsigned & errorCode)
 {
   if (param.m_sessionID != rtp.GetSessionID()) {
     PTRACE(1, "RTP_UDP\tOpen of " << channel << " with invalid session: " << param.m_sessionID);
     errorCode = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
-    return FALSE;
+    return PFalse;
   }
 
-  BOOL ok = FALSE;
+  PBoolean ok = PFalse;
 
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel)) {
-    if (!ExtractTransport(param.m_mediaControlChannel, FALSE, errorCode)) {
+    if (!ExtractTransport(param.m_mediaControlChannel, PFalse, errorCode)) {
       PTRACE(1, "RTP_UDP\tFailed to extract mediaControl transport for " << channel);
-      return FALSE;
+      return PFalse;
     }
-    ok = TRUE;
+    ok = PTrue;
   }
 
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaChannel)) {
     if (ok && channel.GetDirection() == H323Channel::IsReceiver)
-      PTRACE(3, "RTP_UDP\tIgnoring media transport for " << channel);
-    else if (!ExtractTransport(param.m_mediaChannel, TRUE, errorCode)) {
+      PTRACE(2, "RTP_UDP\tIgnoring media transport for " << channel);
+    else if (!ExtractTransport(param.m_mediaChannel, PTrue, errorCode)) {
       PTRACE(1, "RTP_UDP\tFailed to extract media transport for " << channel);
-      return FALSE;
+      return PFalse;
     }
-    ok = TRUE;
+    ok = PTrue;
   }
 
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_dynamicRTPPayloadType))
     channel.SetDynamicRTPPayloadType(param.m_dynamicRTPPayloadType);
 
+  if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaPacketization) &&
+      param.m_mediaPacketization.GetTag() == H245_H2250LogicalChannelParameters_mediaPacketization::e_rtpPayloadType) {
+    OpalMediaFormat mediaFormat = channel.GetMediaStream()->GetMediaFormat();
+    if (H323GetRTPPacketization(mediaFormat, param.m_mediaPacketization))
+      channel.GetMediaStream()->UpdateMediaFormat(mediaFormat);
+  }
+
   if (ok)
-    return TRUE;
+    return PTrue;
 
   PTRACE(1, "RTP_UDP\tNo mediaChannel or mediaControlChannel specified for " << channel);
+
+  if (rtp.GetSessionID() == OpalMediaFormat::DefaultDataSessionID)
+    return PTrue;
+
   errorCode = H245_OpenLogicalChannelReject_cause::e_unspecified;
-  return FALSE;
+  return PFalse;
 }
 
 
-BOOL H323_RTP_UDP::OnReceivedAckPDU(H323_RTPChannel & channel,
+PBoolean H323_RTP_UDP::OnReceivedAckPDU(H323_RTPChannel & channel,
                                     const H245_H2250LogicalChannelAckParameters & param)
 {
   if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID)) {
@@ -342,27 +236,29 @@ BOOL H323_RTP_UDP::OnReceivedAckPDU(H323_RTPChannel & channel,
     PTRACE(1, "RTP_UDP\tAck for invalid session: " << param.m_sessionID);
   }
 
+  unsigned errorCode;
+
   if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel)) {
     PTRACE(1, "RTP_UDP\tNo mediaControlChannel specified");
-    return FALSE;
+    if (rtp.GetSessionID() != OpalMediaFormat::DefaultDataSessionID)
+      return PFalse;
   }
-
-  unsigned errorCode;
-  if (!ExtractTransport(param.m_mediaControlChannel, FALSE, errorCode))
-    return FALSE;
+  else
+  if (!ExtractTransport(param.m_mediaControlChannel, PFalse, errorCode))
+    return PFalse;
 
   if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel)) {
     PTRACE(1, "RTP_UDP\tNo mediaChannel specified");
-    return FALSE;
+    return PFalse;
   }
 
-  if (!ExtractTransport(param.m_mediaChannel, TRUE, errorCode))
-    return FALSE;
+  if (!ExtractTransport(param.m_mediaChannel, PTrue, errorCode))
+    return PFalse;
 
   if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_dynamicRTPPayloadType))
     channel.SetDynamicRTPPayloadType(param.m_dynamicRTPPayloadType);
 
-  return TRUE;
+  return PTrue;
 }
 
 
@@ -384,4 +280,6 @@ void H323_RTP_UDP::OnSendRasInfo(H225_RTPSession & info)
 }
 
 
+#endif // OPAL_H323
+
 /////////////////////////////////////////////////////////////////////////////
