@@ -23,139 +23,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: peclient.cxx,v $
- * Revision 2.2  2004/06/04 06:54:18  csoutheren
- * Migrated updates from OpenH323 1.14.1
- *
- * Revision 2.1  2004/02/19 10:47:05  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 1.39  2003/05/14 03:06:22  rjongbloed
- * Added another method for removing service relationships.
- * Added virtual for handling SR requests.
- *
- * Revision 1.38  2003/05/05 08:28:25  craigs
- * Fixed lastUpdate time in descriptors
- *
- * Revision 1.37  2003/04/30 07:32:55  craigs
- * Improve handling of wildcard matches
- *
- * Revision 1.36  2003/04/30 04:55:41  craigs
- * Improved handling for nonexistent routes
- *
- * Revision 1.35  2003/04/18 15:16:43  craigs
- * Fixed problem with creation of alias keys
- *
- * Revision 1.34  2003/04/10 14:34:05  craigs
- * Fixed wild card handling
- *
- * Revision 1.33  2003/04/10 09:41:02  robertj
- * Added some more functions for converting to alias addresses.
- *
- * Revision 1.32  2003/04/10 07:05:35  craigs
- * Allowed access to endpoint type in descriptors
- *
- * Revision 1.31  2003/04/10 03:41:58  craigs
- * Allow AccessRequest to return multiple transport addresses
- *
- * Revision 1.30  2003/04/10 00:59:35  craigs
- * Added support for multiple contact addresses per template
- *
- * Revision 1.29  2003/04/09 10:47:06  craigs
- * Fixed problems
- *
- * Revision 1.28  2003/04/09 03:08:10  robertj
- * Fixed race condition in shutting down transactor (pure virtual call)
- *
- * Revision 1.27  2003/04/08 12:23:37  craigs
- * Fixed problem with descriptors not being removed when service relationships go away
- *
- * Revision 1.26  2003/04/07 05:10:50  craigs
- * Added changes to get access to descriptor creates/updates/deletes
- *
- * Revision 1.25  2003/04/02 06:06:01  robertj
- * Added versions of AddDescriptor that contain the GUID.
- * Changed default localIdentifier to be the local username of the endpoint.
- *
- * Revision 1.24  2003/04/01 05:59:33  robertj
- * Fixed H.501 transaction code setting members for m_common PDU part.
- *
- * Revision 1.23  2003/04/01 04:47:55  robertj
- * Abstracted H.225 RAS transaction processing (RIP and secondary thread) in
- *   server environment for use by H.501 peer elements.
- *
- * Revision 1.22  2003/04/01 01:18:16  robertj
- * Minor changes to AccessEquest and AddDescriptor API to allow for
- *   string arrays of aliases.
- *
- * Revision 1.21  2003/03/28 04:43:05  craigs
- * Added noCallSpecific flag for compatibility
- *
- * Revision 1.20  2003/03/28 00:30:13  craigs
- * Fixed problems with service relationship ordinals and better descriptor update access
- *
- * Revision 1.19  2003/03/27 09:23:41  craigs
- * Rewritten support for descriptors and multiple templates
- *
- * Revision 1.18  2003/03/26 00:46:29  robertj
- * Had another go at making H323Transactor being able to be created
- *   without having a listener running.
- *
- * Revision 1.17  2003/03/25 12:53:00  robertj
- * Added SetPromiscuous back in
- *
- * Revision 1.16  2003/03/25 12:01:30  craigs
- * Fixed SEGV when no interface specified for peer element
- *
- * Revision 1.15  2003/03/25 07:50:23  craigs
- * Added support for mutiple transports per descriptor
- *
- * Revision 1.14  2003/03/25 05:13:00  craigs
- * More speed enhancements
- *
- * Revision 1.13  2003/03/25 02:57:04  craigs
- * Fixed for update problems
- *
- * Revision 1.12  2003/03/25 01:59:13  robertj
- * Fixed incorrect position of delete. Would do nothing there!
- *
- * Revision 1.11  2003/03/25 01:41:02  craigs
- * Still more signficant H.501 updates
- *
- * Revision 1.10  2003/03/20 01:51:12  robertj
- * More abstraction of H.225 RAS and H.501 protocols transaction handling.
- *
- * Revision 1.9  2003/03/19 01:18:38  robertj
- * Fixed GNU warnings
- *
- * Revision 1.8  2003/03/19 01:11:37  robertj
- * GNU compatibility
- *
- * Revision 1.7  2003/03/18 13:57:53  craigs
- * More H.501 implementation
- *
- * Revision 1.6  2003/03/17 13:19:31  craigs
- * More H501 implementation
- *
- * Revision 1.5  2003/03/14 06:01:16  craigs
- * More updates
- *
- * Revision 1.4  2003/03/01 00:22:26  craigs
- * New PeerElement implementation
- *
- * Revision 1.3  2003/02/25 06:48:19  robertj
- * More work on PDU transaction abstraction.
- *
- * Revision 1.2  2003/02/21 07:23:18  robertj
- * Fixed up some comments
- *
- * Revision 1.1  2003/02/21 05:27:06  craigs
- * Initial version
- *
+ * $Revision: 19796 $
+ * $Author: rjongbloed $
+ * $Date: 2008-03-25 02:36:39 +0100 (mar 25 mar 2008) $
  */
 
 #include <ptlib.h>
 
+#include <opal/buildopts.h>
+#if OPAL_H323
+
 #ifdef __GNUC__
 #pragma implementation "peclient.h"
 #endif
@@ -177,7 +54,7 @@ const unsigned ServiceRelationshipTimeToLive = 60;
 
 ////////////////////////////////////////////////////////////////
 
-H501Transaction::H501Transaction(H323PeerElement & pe, const H501PDU & pdu, BOOL hasReject)
+H501Transaction::H501Transaction(H323PeerElement & pe, const H501PDU & pdu, PBoolean hasReject)
 : H323Transaction(pe, pdu, new H501PDU, hasReject ? new H501PDU : NULL),
     requestCommon(((H501PDU &)request->GetPDU()).m_common),
     confirmCommon(((H501PDU &)confirm->GetPDU()).m_common),
@@ -206,7 +83,7 @@ H235Authenticator::ValidationResult H501Transaction::ValidatePDU() const
 
 H501ServiceRequest::H501ServiceRequest(H323PeerElement & pe,
                                        const H501PDU & pdu)
-  : H501Transaction(pe, pdu, TRUE),
+  : H501Transaction(pe, pdu, PTrue),
     srq((H501_ServiceRequest &)request->GetChoice().GetObject()),
     scf(((H501PDU &)confirm->GetPDU()).BuildServiceConfirmation(pdu.m_common.m_sequenceNumber)),
     srj(((H501PDU &)reject->GetPDU()).BuildServiceRejection(pdu.m_common.m_sequenceNumber,
@@ -239,7 +116,7 @@ H323Transaction::Response H501ServiceRequest::OnHandlePDU()
 
 H501DescriptorUpdate::H501DescriptorUpdate(H323PeerElement & pe,
                                            const H501PDU & pdu)
-  : H501Transaction(pe, pdu, FALSE),
+  : H501Transaction(pe, pdu, PFalse),
     du((H501_DescriptorUpdate &)request->GetChoice().GetObject()),
     ack(((H501PDU &)confirm->GetPDU()).BuildDescriptorUpdateAck(pdu.m_common.m_sequenceNumber))
 {
@@ -270,7 +147,7 @@ H323Transaction::Response H501DescriptorUpdate::OnHandlePDU()
 
 H501AccessRequest::H501AccessRequest(H323PeerElement & pe,
                                      const H501PDU & pdu)
-  : H501Transaction(pe, pdu, TRUE),
+  : H501Transaction(pe, pdu, PTrue),
     arq((H501_AccessRequest &)request->GetChoice().GetObject()),
     acf(((H501PDU &)confirm->GetPDU()).BuildAccessConfirmation(pdu.m_common.m_sequenceNumber)),
     arj(((H501PDU &)reject->GetPDU()).BuildAccessRejection(pdu.m_common.m_sequenceNumber,
@@ -320,22 +197,19 @@ void H323PeerElement::Construct()
   if (transport != NULL)
     transport->SetPromiscuous(H323Transport::AcceptFromAny);
 
-  monitorStop       = FALSE;
+  monitorStop       = PFalse;
   localIdentifier   = endpoint.GetLocalUserName();
   basePeerOrdinal   = RemoteServiceRelationshipOrdinal;
 
   StartChannel();
 
-  monitor = PThread::Create(PCREATE_NOTIFIER(MonitorMain), 0,
-                            PThread::NoAutoDeleteThread,
-                            PThread::NormalPriority,
-                            "PeerElementMonitor:%x");
+  monitor = PThread::Create(PCREATE_NOTIFIER(MonitorMain), "PeerElementMonitor");
 }
 
 H323PeerElement::~H323PeerElement()
 {
   if (monitor != NULL) {
-    monitorStop = TRUE;
+    monitorStop = PTrue;
     monitorTickle.Signal();
     monitor->WaitForTermination();
     delete monitor;
@@ -379,7 +253,7 @@ void H323PeerElement::PrintOn(ostream & strm) const
 
 void H323PeerElement::MonitorMain(PThread &, INT)
 {
-  PTRACE(3, "PeerElement\tBackground thread started");
+  PTRACE(4, "PeerElement\tBackground thread started");
 
   for (;;) {
 
@@ -407,7 +281,7 @@ void H323PeerElement::MonitorMain(PThread &, INT)
         // check to see if expired or needs refresh scheduled
         PTime expireTime = sr->expireTime + 1000 * ServiceRequestGracePeriod;
         if (now >= expireTime) {
-          PTRACE(3, "PeerElement\tService relationship " << sr->serviceID << "expired");
+          PTRACE(2, "PeerElement\tService relationship " << sr->serviceID << "expired");
           localServiceRelationships.Remove(sr);
           {
             PWaitAndSignal m(localPeerListMutex);
@@ -430,7 +304,7 @@ void H323PeerElement::MonitorMain(PThread &, INT)
               !localServiceOrdinals.Contains(descriptor->creator)
              )
             ) {
-          PThread::Create(PCREATE_NOTIFIER(UpdateAllDescriptors), 0, PThread::AutoDeleteThread, PThread::NormalPriority);
+          PThread::Create(PCREATE_NOTIFIER(UpdateAllDescriptors), 0, PThread::AutoDeleteThread, PThread::NormalPriority, "UpdateDescriptors");
           break;
         }
       }
@@ -446,12 +320,12 @@ void H323PeerElement::MonitorMain(PThread &, INT)
       break;
   }
 
-  PTRACE(3, "PeerElement\tBackground thread ended");
+  PTRACE(4, "PeerElement\tBackground thread ended");
 }
 
 void H323PeerElement::UpdateAllDescriptors(PThread &, INT)
 {
-  PTRACE(2, "PeerElement\tDescriptor update thread started");
+  PTRACE(4, "PeerElement\tDescriptor update thread started");
 
   for (PSafePtr<H323PeerElementDescriptor> descriptor = GetFirstDescriptor(PSafeReadWrite); descriptor != NULL; descriptor++) {
     PWaitAndSignal m(localPeerListMutex);
@@ -469,7 +343,7 @@ void H323PeerElement::UpdateAllDescriptors(PThread &, INT)
 
   monitorTickle.Signal();
 
-  PTRACE(2, "PeerElement\tDescriptor update thread ended");
+  PTRACE(4, "PeerElement\tDescriptor update thread ended");
 }
 
 void H323PeerElement::TickleMonitor(PTimer &, INT)
@@ -487,11 +361,11 @@ H323PeerElementServiceRelationship * H323PeerElement::CreateServiceRelationship(
   return new H323PeerElementServiceRelationship();
 }
 
-BOOL H323PeerElement::SetOnlyServiceRelationship(const PString & peer, BOOL keepTrying)
+PBoolean H323PeerElement::SetOnlyServiceRelationship(const PString & peer, PBoolean keepTrying)
 {
   if (peer.IsEmpty()) {
     RemoveAllServiceRelationships();
-    return TRUE;
+    return PTrue;
   }
 
   for (PSafePtr<H323PeerElementServiceRelationship> sr = GetFirstRemoteServiceRelationship(PSafeReadOnly); sr != NULL; sr++)
@@ -501,29 +375,29 @@ BOOL H323PeerElement::SetOnlyServiceRelationship(const PString & peer, BOOL keep
   return AddServiceRelationship(peer, keepTrying);
 }
 
-BOOL H323PeerElement::AddServiceRelationship(const H323TransportAddress & addr, BOOL keepTrying)
+PBoolean H323PeerElement::AddServiceRelationship(const H323TransportAddress & addr, PBoolean keepTrying)
 {
   OpalGloballyUniqueID serviceID;
   return AddServiceRelationship(addr, serviceID, keepTrying);
 }
 
-BOOL H323PeerElement::AddServiceRelationship(const H323TransportAddress & addr, OpalGloballyUniqueID & serviceID, BOOL keepTrying)
+PBoolean H323PeerElement::AddServiceRelationship(const H323TransportAddress & addr, OpalGloballyUniqueID & serviceID, PBoolean keepTrying)
 
 {
   switch (ServiceRequestByAddr(addr, serviceID)) {
     case Confirmed:
     case ServiceRelationshipReestablished:
-      return TRUE;
+      return PTrue;
 
     case NoResponse:
       if (!keepTrying)
-        return FALSE;
+        return PFalse;
       break;    
     
     case Rejected:
     case NoServiceRelationship:
     default:
-      return FALSE;
+      return PFalse;
   }
 
   PTRACE(2, "PeerElement\tRetrying ServiceRequest to " << addr << " in " << ServiceRequestRetryTime);
@@ -545,10 +419,10 @@ BOOL H323PeerElement::AddServiceRelationship(const H323TransportAddress & addr,
 
   monitorTickle.Signal();
 
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::RemoveServiceRelationship(const OpalGloballyUniqueID & serviceID, int reason)
+PBoolean H323PeerElement::RemoveServiceRelationship(const OpalGloballyUniqueID & serviceID, int reason)
 {
   {
     PWaitAndSignal m(remotePeerListMutex);
@@ -556,7 +430,7 @@ BOOL H323PeerElement::RemoveServiceRelationship(const OpalGloballyUniqueID & ser
     // if no service relationship exists for this peer, then nothing to do
     PSafePtr<H323PeerElementServiceRelationship> sr = remoteServiceRelationships.FindWithLock(H323PeerElementServiceRelationship(serviceID), PSafeReadOnly);
     if (sr == NULL) {
-      return FALSE;
+      return PFalse;
     }
   }
 
@@ -564,7 +438,7 @@ BOOL H323PeerElement::RemoveServiceRelationship(const OpalGloballyUniqueID & ser
 }
 
 
-BOOL H323PeerElement::RemoveServiceRelationship(const H323TransportAddress & peer, int reason)
+PBoolean H323PeerElement::RemoveServiceRelationship(const H323TransportAddress & peer, int reason)
 {
   OpalGloballyUniqueID serviceID;
 
@@ -572,20 +446,20 @@ BOOL H323PeerElement::RemoveServiceRelationship(const H323TransportAddress & pee
   {
     PWaitAndSignal m(remotePeerListMutex);
     if (!remotePeerAddrToServiceID.Contains(peer))
-      return FALSE;
+      return PFalse;
     serviceID = remotePeerAddrToServiceID[peer];
   }
 
   return ServiceRelease(serviceID, reason);
 }
 
-BOOL H323PeerElement::RemoveAllServiceRelationships()
+PBoolean H323PeerElement::RemoveAllServiceRelationships()
 {
   // if a service relationship exists for this peer, then reconfirm it
   for (PSafePtr<H323PeerElementServiceRelationship> sr = GetFirstRemoteServiceRelationship(PSafeReadOnly); sr != NULL; sr++)
     RemoveServiceRelationship(sr->peer);
 
-  return TRUE;
+  return PTrue;
 }
 
 H323PeerElement::Error H323PeerElement::ServiceRequestByAddr(const H323TransportAddress & peer, OpalGloballyUniqueID & serviceID)
@@ -667,7 +541,7 @@ H323PeerElement::Error H323PeerElement::ServiceRequestByAddr(const H323Transport
 
   remoteServiceRelationships.Append(sr);
 
-  PTRACE(2, "PeerElement\tNew service relationship established with " << peer << " - next update in " << replyBody.m_timeToLive);
+  PTRACE(3, "PeerElement\tNew service relationship established with " << peer << " - next update in " << replyBody.m_timeToLive);
   OnAddServiceRelationship(peer);
 
   // mark all descriptors as needing an update
@@ -710,7 +584,7 @@ H323PeerElement::Error H323PeerElement::ServiceRequestByID(OpalGloballyUniqueID
     H501_ServiceConfirmation & replyBody = reply.m_body;
     sr->expireTime = PTime() + 1000 * ((replyBody.m_timeToLive < ServiceRequestRetryTime) ? (int)replyBody.m_timeToLive : ServiceRequestRetryTime);
     sr->lastUpdateTime = PTime();
-    PTRACE(2, "PeerElement\tConfirmed service relationship with " << sr->peer << " - next update in " << replyBody.m_timeToLive);
+    PTRACE(3, "PeerElement\tConfirmed service relationship with " << sr->peer << " - next update in " << replyBody.m_timeToLive);
     return Confirmed;
   }
 
@@ -758,7 +632,7 @@ H323Transaction::Response H323PeerElement::HandleServiceRequest(H501ServiceReque
     OpalGloballyUniqueID serviceID(info.requestCommon.m_serviceID);
     PSafePtr<H323PeerElementServiceRelationship> sr = localServiceRelationships.FindWithLock(H323PeerElementServiceRelationship(serviceID), PSafeReadWrite);
     if (sr == NULL) {
-      PTRACE(2, "PeerElement\nRejecting unknown service ID " << serviceID << " received from peer " << info.GetReplyAddress());
+      PTRACE(2, "PeerElement\tRejecting unknown service ID " << serviceID << " received from peer " << info.GetReplyAddress());
       info.SetRejectReason(H501_ServiceRejectionReason::e_unknownServiceID);
       return H323Transaction::Reject;
     }
@@ -775,7 +649,7 @@ H323Transaction::Response H323PeerElement::HandleServiceRequest(H501ServiceReque
     sr->lastUpdateTime = PTime();
     sr->expireTime = PTime() + (info.scf.m_timeToLive * 1000);
 
-    PTRACE(2, "PeerElement\nService relationship with " << sr->name << " at " << info.GetReplyAddress() << " updated - next update in " << info.scf.m_timeToLive);
+    PTRACE(2, "PeerElement\tService relationship with " << sr->name << " at " << info.GetReplyAddress() << " updated - next update in " << info.scf.m_timeToLive);
     return H323Transaction::Confirm;
   }
 
@@ -817,36 +691,36 @@ H323Transaction::Response H323PeerElement::HandleServiceRequest(H501ServiceReque
   monitorTickle.Signal();
 
   // send the response
-  PTRACE(2, "PeerElement\nNew service relationship with " << sr->name << " at " << info.GetReplyAddress() << " created - next update in " << info.scf.m_timeToLive);
+  PTRACE(3, "PeerElement\tNew service relationship with " << sr->name << " at " << info.GetReplyAddress() << " created - next update in " << info.scf.m_timeToLive);
   return H323Transaction::Confirm;
 }
 
-BOOL H323PeerElement::OnReceiveServiceRequest(const H501PDU & pdu, const H501_ServiceRequest & /*pduBody*/)
+PBoolean H323PeerElement::OnReceiveServiceRequest(const H501PDU & pdu, const H501_ServiceRequest & /*pduBody*/)
 {
   H501ServiceRequest * info = new H501ServiceRequest(*this, pdu);
   if (!info->HandlePDU())
     delete info;
 
-  return FALSE;
+  return PFalse;
 }
 
-BOOL H323PeerElement::OnReceiveServiceConfirmation(const H501PDU & pdu, const H501_ServiceConfirmation & pduBody)
+PBoolean H323PeerElement::OnReceiveServiceConfirmation(const H501PDU & pdu, const H501_ServiceConfirmation & pduBody)
 {
   if (!H323_AnnexG::OnReceiveServiceConfirmation(pdu, pduBody))
-    return FALSE;
+    return PFalse;
 
   if (lastRequest->responseInfo != NULL)
     *(H501PDU *)lastRequest->responseInfo = pdu;
 
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::ServiceRelease(const OpalGloballyUniqueID & serviceID, unsigned reason)
+PBoolean H323PeerElement::ServiceRelease(const OpalGloballyUniqueID & serviceID, unsigned reason)
 {
   // remove any previous check to see if we have a service relationship with the peer already
   PSafePtr<H323PeerElementServiceRelationship> sr = remoteServiceRelationships.FindWithLock(H323PeerElementServiceRelationship(serviceID), PSafeReadWrite);
   if (sr == NULL)
-    return FALSE;
+    return PFalse;
 
   // send the request - no response
   H501PDU pdu;
@@ -859,10 +733,10 @@ BOOL H323PeerElement::ServiceRelease(const OpalGloballyUniqueID & serviceID, uns
   InternalRemoveServiceRelationship(sr->peer);
   remoteServiceRelationships.Remove(sr);
 
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::OnRemoteServiceRelationshipDisappeared(OpalGloballyUniqueID & serviceID, const H323TransportAddress & peer)
+PBoolean H323PeerElement::OnRemoteServiceRelationshipDisappeared(OpalGloballyUniqueID & serviceID, const H323TransportAddress & peer)
 {
   OpalGloballyUniqueID oldServiceID = serviceID;
 
@@ -876,14 +750,14 @@ BOOL H323PeerElement::OnRemoteServiceRelationshipDisappeared(OpalGloballyUniqueI
   if (ServiceRequestByAddr(peer, serviceID) != Confirmed) { 
     PTRACE(2, "PeerElement\tService relationship with " << peer << " disappeared and refused new relationship");
     OnRemoveServiceRelationship(peer);
-    return FALSE;
+    return PFalse;
   }
 
   // we have a new service ID
   PTRACE(2, "PeerElement\tService relationship with " << peer << " disappeared and new relationship established");
   serviceID = remotePeerAddrToServiceID(peer);
 
-  return TRUE;
+  return PTrue;
 }
 
 void H323PeerElement::InternalRemoveServiceRelationship(const H323TransportAddress & peer)
@@ -908,11 +782,11 @@ H323PeerElementDescriptor * H323PeerElement::CreateDescriptor(const OpalGlobally
   return new H323PeerElementDescriptor(descriptorID);
 }
 
-BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
+PBoolean H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
                                             const PStringArray & aliasStrings, 
                                const H323TransportAddressArray & transportAddresses, 
                                                         unsigned options, 
-                                                            BOOL now)
+                                                            PBoolean now)
 {
   // convert transport addresses to aliases
   H225_ArrayOf_AliasAddress aliases;
@@ -922,11 +796,11 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
 }
 
 
-BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
+PBoolean H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
                                const H225_ArrayOf_AliasAddress & aliases, 
                                const H323TransportAddressArray & transportAddresses, 
                                                         unsigned options, 
-                                                            BOOL now)
+                                                            PBoolean now)
 {
   H225_ArrayOf_AliasAddress addresses;
   H323SetAliasAddresses(transportAddresses, addresses);
@@ -936,11 +810,11 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
 }
 
 
-BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
+PBoolean H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
                                const H225_ArrayOf_AliasAddress & aliases, 
                                const H225_ArrayOf_AliasAddress & transportAddress, 
                                                         unsigned options, 
-                                                            BOOL now)
+                                                            PBoolean now)
 {
   // create a new descriptor
   return AddDescriptor(descriptorID,
@@ -948,12 +822,12 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
                        aliases, transportAddress, options, now);
 }
 
-BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
+PBoolean H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
                                              const POrdinalKey & creator,
                                const H225_ArrayOf_AliasAddress & aliases, 
                                const H225_ArrayOf_AliasAddress & transportAddresses, 
                                                         unsigned options, 
-                                                            BOOL now)
+                                                            PBoolean now)
 {
   // create an const H501_ArrayOf_AddressTemplate with the template information
   H501_ArrayOf_AddressTemplate addressTemplates;
@@ -967,16 +841,16 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
   return AddDescriptor(descriptorID, creator, addressTemplates, now);
 }
 
-BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
+PBoolean H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
                                              const POrdinalKey & creator,
                             const H501_ArrayOf_AddressTemplate & addressTemplates,
                                                    const PTime & updateTime,
-                                                            BOOL now)
+                                                            PBoolean now)
 {
   // see if there is actually a descriptor with this ID
   PSafePtr<H323PeerElementDescriptor> descriptor = descriptors.FindWithLock(H323PeerElementDescriptor(descriptorID), PSafeReadWrite);
   H501_UpdateInformation_updateType::Choices updateType = H501_UpdateInformation_updateType::e_changed;
-  BOOL add = FALSE;
+  PBoolean add = PFalse;
   {
     PWaitAndSignal m(aliasMutex);
     if (descriptor != NULL) {
@@ -984,10 +858,10 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
 
       // only update if the update time is later than what we already have
       if (updateTime < descriptor->lastChanged)
-        return TRUE;
+        return PTrue;
 
     } else {
-      add = TRUE;
+      add = PTrue;
       descriptor                   = CreateDescriptor(descriptorID);
       descriptor->creator          = creator;
       descriptor->addressTemplates = addressTemplates;
@@ -1005,10 +879,10 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
         H501_Pattern & pattern = addressTemplate.m_pattern[j];
         switch (pattern.GetTag()) {
           case H501_Pattern::e_specific:
-            specificAliasToDescriptorID.Append(CreateAliasKey((H225_AliasAddress &)pattern, descriptorID, i, FALSE));
+            specificAliasToDescriptorID.Append(CreateAliasKey((H225_AliasAddress &)pattern, descriptorID, i, PFalse));
             break;
           case H501_Pattern::e_wildcard:
-            wildcardAliasToDescriptorID.Append(CreateAliasKey((H225_AliasAddress &)pattern, descriptorID, i, TRUE));
+            wildcardAliasToDescriptorID.Append(CreateAliasKey((H225_AliasAddress &)pattern, descriptorID, i, PTrue));
             break;
           case H501_Pattern::e_range:
             break;
@@ -1037,18 +911,18 @@ BOOL H323PeerElement::AddDescriptor(const OpalGloballyUniqueID & descriptorID,
 
   // do the update now, or later
   if (now) {
-    PTRACE(2, "PeerElement\tDescriptor " << descriptorID << " added/updated");
+    PTRACE(3, "PeerElement\tDescriptor " << descriptorID << " added/updated");
     UpdateDescriptor(descriptor, updateType);
   } else if (descriptor->state != H323PeerElementDescriptor::Deleted) {
-    PTRACE(2, "PeerElement\tDescriptor " << descriptorID << " queued to be added");
+    PTRACE(3, "PeerElement\tDescriptor " << descriptorID << " queued to be added");
     descriptor->state = H323PeerElementDescriptor::Dirty;
     monitorTickle.Signal();
   }
 
-  return TRUE;
+  return PTrue;
 }
   
-H323PeerElement::AliasKey * H323PeerElement::CreateAliasKey(const H225_AliasAddress & alias, const OpalGloballyUniqueID & id, PINDEX pos, BOOL wild)
+H323PeerElement::AliasKey * H323PeerElement::CreateAliasKey(const H225_AliasAddress & alias, const OpalGloballyUniqueID & id, PINDEX pos, PBoolean wild)
 {
   return new AliasKey(alias, id, pos, wild);
 }
@@ -1096,14 +970,14 @@ void H323PeerElement::RemoveDescriptorInformation(const H501_ArrayOf_AddressTemp
   }
 }
 
-BOOL H323PeerElement::DeleteDescriptor(const PString & str, BOOL now)
+PBoolean H323PeerElement::DeleteDescriptor(const PString & str, PBoolean now)
 {
   H225_AliasAddress alias;
   H323SetAliasAddress(str, alias);
   return DeleteDescriptor(alias, now);
 }
 
-BOOL H323PeerElement::DeleteDescriptor(const H225_AliasAddress & alias, BOOL now)
+PBoolean H323PeerElement::DeleteDescriptor(const H225_AliasAddress & alias, PBoolean now)
 {
   OpalGloballyUniqueID descriptorID("");
 
@@ -1112,19 +986,19 @@ BOOL H323PeerElement::DeleteDescriptor(const H225_AliasAddress & alias, BOOL now
     PWaitAndSignal m(aliasMutex);
     PINDEX idx = specificAliasToDescriptorID.GetValuesIndex(alias);
     if (idx == P_MAX_INDEX)
-      return FALSE;
+      return PFalse;
     descriptorID = ((AliasKey &)specificAliasToDescriptorID[idx]).id;
   }
 
   return DeleteDescriptor(descriptorID, now);
 }
 
-BOOL H323PeerElement::DeleteDescriptor(const OpalGloballyUniqueID & descriptorID, BOOL now)
+PBoolean H323PeerElement::DeleteDescriptor(const OpalGloballyUniqueID & descriptorID, PBoolean now)
 {
   // see if there is a descriptor with this ID
   PSafePtr<H323PeerElementDescriptor> descriptor = descriptors.FindWithLock(H323PeerElementDescriptor(descriptorID), PSafeReadWrite);
   if (descriptor == NULL)
-    return FALSE;
+    return PFalse;
 
   OnRemoveDescriptor(*descriptor);
 
@@ -1132,23 +1006,23 @@ BOOL H323PeerElement::DeleteDescriptor(const OpalGloballyUniqueID & descriptorID
 
   // delete the descriptor, or mark it as to be deleted
   if (now) {
-    PTRACE(2, "PeerElement\tDescriptor " << descriptorID << " deleted");
+    PTRACE(3, "PeerElement\tDescriptor " << descriptorID << " deleted");
     UpdateDescriptor(descriptor, H501_UpdateInformation_updateType::e_deleted);
   } else {
-    PTRACE(2, "PeerElement\tDescriptor for " << descriptorID << " queued to be deleted");
+    PTRACE(3, "PeerElement\tDescriptor for " << descriptorID << " queued to be deleted");
     descriptor->state = H323PeerElementDescriptor::Deleted;
     monitorTickle.Signal();
   }
 
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::UpdateDescriptor(H323PeerElementDescriptor * descriptor)
+PBoolean H323PeerElement::UpdateDescriptor(H323PeerElementDescriptor * descriptor)
 {
   H501_UpdateInformation_updateType::Choices updateType = H501_UpdateInformation_updateType::e_changed;
   switch (descriptor->state) {
     case H323PeerElementDescriptor::Clean:
-      return TRUE;
+      return PTrue;
     
     case H323PeerElementDescriptor::Dirty:
       break;
@@ -1161,14 +1035,14 @@ BOOL H323PeerElement::UpdateDescriptor(H323PeerElementDescriptor * descriptor)
   return UpdateDescriptor(descriptor, updateType);
 }
 
-BOOL H323PeerElement::UpdateDescriptor(H323PeerElementDescriptor * descriptor, H501_UpdateInformation_updateType::Choices updateType)
+PBoolean H323PeerElement::UpdateDescriptor(H323PeerElementDescriptor * descriptor, H501_UpdateInformation_updateType::Choices updateType)
 {
   if (updateType == H501_UpdateInformation_updateType::e_deleted)
     descriptor->state = H323PeerElementDescriptor::Deleted;
   else if (descriptor->state == H323PeerElementDescriptor::Deleted)
     updateType = H501_UpdateInformation_updateType::e_deleted;
   else if (descriptor->state == H323PeerElementDescriptor::Clean)
-    return TRUE;
+    return PTrue;
   else
     descriptor->state = H323PeerElementDescriptor::Clean;
 
@@ -1179,7 +1053,7 @@ BOOL H323PeerElement::UpdateDescriptor(H323PeerElementDescriptor * descriptor, H
   if (descriptor->state == H323PeerElementDescriptor::Deleted)
     descriptors.Remove(descriptor);
 
-  return TRUE;
+  return PTrue;
 }
 
 ///////////////////////////////////////////////////////////
@@ -1273,24 +1147,24 @@ H323Transaction::Response H323PeerElement::OnDescriptorUpdate(H501DescriptorUpda
   return H323Transaction::Ignore;
 }
 
-BOOL H323PeerElement::OnReceiveDescriptorUpdate(const H501PDU & pdu, const H501_DescriptorUpdate & /*pduBody*/)
+PBoolean H323PeerElement::OnReceiveDescriptorUpdate(const H501PDU & pdu, const H501_DescriptorUpdate & /*pduBody*/)
 {
   H501DescriptorUpdate * info = new H501DescriptorUpdate(*this, pdu);
   if (!info->HandlePDU())
     delete info;
 
-  return FALSE;
+  return PFalse;
 }
 
-BOOL H323PeerElement::OnReceiveDescriptorUpdateACK(const H501PDU & pdu, const H501_DescriptorUpdateAck & pduBody)
+PBoolean H323PeerElement::OnReceiveDescriptorUpdateACK(const H501PDU & pdu, const H501_DescriptorUpdateAck & pduBody)
 {
   if (!H323_AnnexG::OnReceiveDescriptorUpdateACK(pdu, pduBody))
-    return FALSE;
+    return PFalse;
 
   if (lastRequest->responseInfo != NULL)
     *(H501_MessageCommonInfo *)lastRequest->responseInfo = pdu.m_common;
 
-  return TRUE;
+  return PTrue;
 }
 
 ///////////////////////////////////////////////////////////
@@ -1298,7 +1172,7 @@ BOOL H323PeerElement::OnReceiveDescriptorUpdateACK(const H501PDU & pdu, const H5
 // access request functions
 //
 
-BOOL H323PeerElement::AccessRequest(const PString & searchAlias,
+PBoolean H323PeerElement::AccessRequest(const PString & searchAlias,
                                      PStringArray & destAliases,
                             H323TransportAddress & transportAddress,
                                           unsigned options)
@@ -1308,14 +1182,14 @@ BOOL H323PeerElement::AccessRequest(const PString & searchAlias,
 
   H225_ArrayOf_AliasAddress h225destAliases;
   if (!AccessRequest(h225searchAlias, h225destAliases, transportAddress, options))
-    return FALSE;
+    return PFalse;
 
   destAliases = H323GetAliasAddressStrings(h225destAliases);
-  return TRUE;
+  return PTrue;
 }
 
 
-BOOL H323PeerElement::AccessRequest(const PString & searchAlias, 
+PBoolean H323PeerElement::AccessRequest(const PString & searchAlias, 
                         H225_ArrayOf_AliasAddress & destAliases,
                              H323TransportAddress & transportAddress, 
                                            unsigned options)
@@ -1325,20 +1199,20 @@ BOOL H323PeerElement::AccessRequest(const PString & searchAlias,
   return AccessRequest(h225searchAlias, destAliases, transportAddress, options);
 }
 
-BOOL H323PeerElement::AccessRequest(const H225_AliasAddress & searchAlias, 
+PBoolean H323PeerElement::AccessRequest(const H225_AliasAddress & searchAlias, 
                                   H225_ArrayOf_AliasAddress & destAliases,
                                        H323TransportAddress & transportAddress, 
                                                      unsigned options)
 {
   H225_AliasAddress h225Address;
   if (!AccessRequest(searchAlias, destAliases, h225Address, options))
-    return FALSE;
+    return PFalse;
 
   transportAddress = H323GetAliasAddressString(h225Address);
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::AccessRequest(const H225_AliasAddress & searchAlias, 
+PBoolean H323PeerElement::AccessRequest(const H225_AliasAddress & searchAlias, 
                                   H225_ArrayOf_AliasAddress & destAliases,
                                           H225_AliasAddress & transportAddress, 
                                                      unsigned options)
@@ -1422,11 +1296,11 @@ BOOL H323PeerElement::AccessRequest(const H225_AliasAddress & searchAlias,
         destAliases.SetSize(count);  
 
         transportAddress = contactAddress;
-        PTRACE(2, "Main\tAccessRequest for " << searchAlias << " returned " << transportAddress << " from " << peerAddr);
-        return TRUE;
+        PTRACE(3, "Main\tAccessRequest for " << searchAlias << " returned " << transportAddress << " from " << peerAddr);
+        return PTrue;
       }
       else { // H501_RouteInformation_messageType::e_nonExistent
-        PTRACE(2, "Main\tAccessRequest for " << searchAlias << " from " << peerAddr << " returned nonExistent");
+        PTRACE(3, "Main\tAccessRequest for " << searchAlias << " from " << peerAddr << " returned nonExistent");
         break;
       }
 
@@ -1450,7 +1324,7 @@ BOOL H323PeerElement::AccessRequest(const H225_AliasAddress & searchAlias,
     }
   }
 
-  return FALSE;
+  return PFalse;
 }
 
 ///////////////////////////////////////////////////////////
@@ -1553,38 +1427,38 @@ H323Transaction::Response H323PeerElement::OnAccessRequest(H501AccessRequest & i
   return H323Transaction::Reject;
 }
 
-BOOL H323PeerElement::OnReceiveAccessRequest(const H501PDU & pdu, const H501_AccessRequest & /*pduBody*/)
+PBoolean H323PeerElement::OnReceiveAccessRequest(const H501PDU & pdu, const H501_AccessRequest & /*pduBody*/)
 {
   H501AccessRequest * info = new H501AccessRequest(*this, pdu);
   if (!info->HandlePDU())
     delete info;
 
-  return FALSE;
+  return PFalse;
 }
 
-BOOL H323PeerElement::OnReceiveAccessConfirmation(const H501PDU & pdu, const H501_AccessConfirmation & pduBody)
+PBoolean H323PeerElement::OnReceiveAccessConfirmation(const H501PDU & pdu, const H501_AccessConfirmation & pduBody)
 {
   if (!H323_AnnexG::OnReceiveAccessConfirmation(pdu, pduBody))
-    return FALSE;
+    return PFalse;
 
   if (lastRequest->responseInfo != NULL)
     *(H501PDU *)lastRequest->responseInfo = pdu;
 
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::OnReceiveAccessRejection(const H501PDU & pdu, const H501_AccessRejection & pduBody)
+PBoolean H323PeerElement::OnReceiveAccessRejection(const H501PDU & pdu, const H501_AccessRejection & pduBody)
 {
   if (!H323_AnnexG::OnReceiveAccessRejection(pdu, pduBody))
-    return FALSE;
+    return PFalse;
 
-  return TRUE;
+  return PTrue;
 }
 
-BOOL H323PeerElement::MakeRequest(Request & request)
+PBoolean H323PeerElement::MakeRequest(Request & request)
 {
   requestMutex.Wait();
-  BOOL stat = H323_AnnexG::MakeRequest(request);
+  PBoolean stat = H323_AnnexG::MakeRequest(request);
   requestMutex.Signal();
   return stat;
 }
@@ -1610,9 +1484,9 @@ void H323PeerElementDescriptor::CopyTo(H501_Descriptor & descriptor)
 }
 
 
-BOOL H323PeerElementDescriptor::ContainsNonexistent()
+PBoolean H323PeerElementDescriptor::ContainsNonexistent()
 {
-  BOOL blocked = FALSE;
+  PBoolean blocked = PFalse;
 
   // look for any nonexistent routes, which means this descriptor does NOT match
   PINDEX k, j;
@@ -1620,7 +1494,7 @@ BOOL H323PeerElementDescriptor::ContainsNonexistent()
 	  H501_ArrayOf_RouteInformation & routeInfo = addressTemplates[k].m_routeInfo;
     for (j = 0; !blocked && (j < routeInfo.GetSize()); j++) {
       if (routeInfo[j].m_messageType.GetTag() == H501_RouteInformation_messageType::e_nonExistent)
-        blocked = TRUE;
+        blocked = PTrue;
     }
   }
 
@@ -1628,7 +1502,7 @@ BOOL H323PeerElementDescriptor::ContainsNonexistent()
 }
 
 
-BOOL H323PeerElementDescriptor::CopyToAddressTemplate(H501_AddressTemplate & addressTemplate,
+PBoolean H323PeerElementDescriptor::CopyToAddressTemplate(H501_AddressTemplate & addressTemplate,
                                                    const H225_EndpointType & epInfo,
                                            const H225_ArrayOf_AliasAddress & aliases, 
                                            const H225_ArrayOf_AliasAddress & transportAddresses, 
@@ -1659,12 +1533,12 @@ BOOL H323PeerElementDescriptor::CopyToAddressTemplate(H501_AddressTemplate & add
 
   else {
     routeInfo.m_messageType.SetTag(H501_RouteInformation_messageType::e_sendSetup);
-    routeInfo.m_callSpecific = FALSE;
+    routeInfo.m_callSpecific = PFalse;
     routeInfo.IncludeOptionalField(H501_RouteInformation::e_type);
     routeInfo.m_type = epInfo;
   }
 
-  routeInfo.m_callSpecific = FALSE;
+  routeInfo.m_callSpecific = PFalse;
   H501_ArrayOf_ContactInformation & contacts = routeInfos[0].m_contacts;
   contacts.SetSize(transportAddresses.GetSize());
   PINDEX i;
@@ -1678,11 +1552,11 @@ BOOL H323PeerElementDescriptor::CopyToAddressTemplate(H501_AddressTemplate & add
   addressTemplate.IncludeOptionalField(H501_AddressTemplate::e_supportedProtocols);
   SetProtocolList(addressTemplate.m_supportedProtocols, options);
 
-  return TRUE;
+  return PTrue;
 }
 
 /*
-BOOL H323PeerElementDescriptor::CopyFrom(const H501_Descriptor & descriptor)
+PBoolean H323PeerElementDescriptor::CopyFrom(const H501_Descriptor & descriptor)
 {
   descriptorID                           = descriptor.m_descriptorInfo.m_descriptorID;
   //lastChanged.AsString("yyyyMMddhhmmss") = descriptor.m_descriptorInfo.m_lastChanged;
@@ -1693,7 +1567,7 @@ BOOL H323PeerElementDescriptor::CopyFrom(const H501_Descriptor & descriptor)
   else
     gatekeeperID = PString::Empty();
 
-  return TRUE;
+  return PTrue;
 }
 */
 
@@ -1745,4 +1619,6 @@ unsigned H323PeerElementDescriptor::GetProtocolList(const H501_ArrayOf_Supported
 }
 
 
+#endif // OPAL_H323
+
 // End of file ////////////////////////////////////////////////////////////////
