@@ -34,10 +34,10 @@
    which extracts the version numbers
  */
 
-#define MAJOR_VERSION 2
-#define MINOR_VERSION 2
-#define BUILD_TYPE    ReleaseCode
-#define BUILD_NUMBER 8
+#define MAJOR_VERSION 3
+#define MINOR_VERSION 3
+#define BUILD_TYPE BetaCode
+#define BUILD_NUMBER 1
 
 
 #endif  // _OPAL_VERSION_H
